/*
 * transducer_cli_header.c
 *
 *  Created on: 16 mai 2013
 *      Author: rj092
 */

#ifdef WIN32
#include "../../Interface_Library/msxe370x_modbus_clientlib.h"
#else
#include <msxe370x_modbus_clientlib.h>
#endif

#include <addidata_console.h>
#include <addidata_network.h>
#include <addidata_dataclient.h>
#include <msxe370x.h>

#include <modbusclient.h>
#include <commonclient.h>

#ifdef WIN32
	#include <windows.h>
	#define sleep(a) Sleep(a*1000)
#else
	#include <unistd.h>
#endif


/** @file modbuscli.c MODBUS/UDP client sample for the MSXE3701
*
* FC16 queries are for setting things, FC3 for fetching values
*
*/

int sample_MX370x__TransducerSetOffsetEx(struct modbus * modbus)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	int i = 0;
	// Table example
	float fOffset[16] = { 1.0, 0.1, 0.2, 0.3, 0.4, 0.5, -1.0, -0.1, -0.2, -0.3, -0.4, -0.5, -0.6, -0.7, -0.8, -0.9 };

	for (i=0; i<16; i++)
		printf("offset channel %d = %f\n", i, fOffset[i]);

	//Remote function call
	ret = modbus_call_MX370x__TransducerSetOffsetEx(modbus, fOffset, &CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerSetOffsetEx" );
}

//---------------------------------------------------------------------------------
int sample_MX370x__getNumberOfChannels(struct modbus * modbus, uint32_t* ChannelNumber)
{
	struct MX370x__getNumberOfChannels_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__getNumberOfChannels(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__getNumberOfChannels") )
		return -1;

	*ChannelNumber = Response.ChannelNumber;
	printf("number of channel(s) on the module: %d\n",Response.ChannelNumber);
	return 0;
}

//---------------------------------------------------------------------------------
int sample_MX370x__getNumberOfChannelsEx(struct modbus * modbus, uint32_t* ChannelNumber)
{
	struct MX370x__getNumberOfChannelsEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__getNumberOfChannelsEx(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__getNumberOfChannelsEx") )
		return -1;

	*ChannelNumber = Response.ChannelNumber;
	printf("number of channel on the module: %d\n",Response.ChannelNumber);
	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerGetAutoRefreshValues(struct modbus * modbus, struct msxe370x_format *format, struct msxe370x_autorefresh_parameters* Params)
{
	struct MX370x__TransducerGetAutoRefreshValues_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__TransducerGetAutoRefreshValues(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerGetAutoRefreshValues") )
		return -1;

	msxe370x_getautorefreshvalues_printf(Params, Response.Value, Params->NbrOfChannel+1, format);
	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerGetAutoRefreshValuesEx(struct modbus * modbus, struct msxe370x_format *format, struct msxe370x_autorefresh_parameters* Params)
{
	struct MX370x__TransducerGetAutoRefreshValuesEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__TransducerGetAutoRefreshValuesEx(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerGetAutoRefreshValuesEx") )
		return -1;

	msxe370x_getautorefreshvalues_printf(Params, Response.Value, Params->NbrOfChannel+1, format);

	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerInitAndStartAutoRefresh(struct modbus * modbus, struct msxe370x_autorefresh_parameters* Params, uint32_t NumberOfChannels)
{

	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerInitAndStartAutoRefresh(modbus,
			Params->TransducerSelection,
			Params->ChannelMask,
			Params->AverageMode,
			Params->AverageValue,
			Params->DivisionFactor,
			Params->TriggerAction,
			Params->HardwareTriggerCount,
			Params->HardwareTriggerFilterTime,
			Params->ByTriggerNbrOfSeqToAcquire,
			Params->DataFormat,
			Params->Option2,
			Params->Option3,
			Params->Option4,
			&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerInitAndStartAutoRefresh" );

}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerInitAndStartAutoRefreshEx(struct modbus * modbus, struct msxe370x_autorefresh_parameters* Params, uint32_t NumberOfChannels)
{

	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerInitAndStartAutoRefreshEx(modbus,
			Params->TransducerSelection,
			Params->ChannelMask,
			Params->AverageMode,
			Params->AverageValue,
			Params->DivisionFactor,
			Params->TriggerAction,
			Params->HardwareTriggerCount,
			Params->HardwareTriggerFilterTime,
			Params->ByTriggerNbrOfSeqToAcquire,
			Params->DataFormat,
			Params->Option2,
			Params->Option3,
			Params->Option4,
			&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerInitAndStartAutoRefreshEx" );

}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerStopAndReleaseAutoRefresh(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerStopAndReleaseAutoRefresh(modbus,Dummy,&CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerStopAndReleaseAutoRefresh");
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerStopAndReleaseAutoRefreshEx(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerStopAndReleaseAutoRefreshEx(modbus,Dummy,&CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerStopAndReleaseAutoRefreshEx");
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerInitAndStartSequence(struct modbus * modbus, struct msxe370x_sequence_parameters* Params, uint32_t NumberOfChannels)
{
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerInitAndStartSequence(modbus,
			Params->TransducerSelection,
			Params->NbrOfChannel,
			Params->ChannelList,
			Params->DivisionFactor,
			Params->NbrOfSequence,
			Params->NbrMaxSequenceToTransfer,
			Params->DelayMode,
			Params->DelayTimeUnit,
			Params->DelayValue,
			Params->TriggerAction,
			Params->HardwareTriggerCount,
			Params->HardwareTriggerFilterTime,
			Params->ByTriggerNbrOfSeqToAcquire,
			Params->DataFormat,
			Params->Option2,
			Params->Option3,
			Params->Option4,
			&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerInitAndStartSequence" );

}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerInitAndStartSequenceEx(struct modbus * modbus, struct msxe370x_sequence_parameters* Params, uint32_t NumberOfChannels)
{
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerInitAndStartSequenceEx(modbus,
			Params->TransducerSelection,
			Params->NbrOfChannel,
			Params->ChannelList,
			Params->DivisionFactor,
			Params->NbrOfSequence,
			Params->NbrMaxSequenceToTransfer,
			Params->DelayMode,
			Params->DelayTimeUnit,
			Params->DelayValue,
			Params->TriggerAction,
			Params->HardwareTriggerCount,
			Params->HardwareTriggerFilterTime,
			Params->ByTriggerNbrOfSeqToAcquire,
			Params->DataFormat,
			Params->Option2,
			Params->Option3,
			Params->Option4,
			&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerInitAndStartSequenceEx" );

}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerStopAndReleaseSequence(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerStopAndReleaseSequence(modbus,Dummy,&CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerStopAndReleaseSequence");
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerStopAndReleaseSequenceEx(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerStopAndReleaseSequenceEx(modbus,Dummy,&CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerStopAndReleaseSequenceEx");
}
//---------------------------------------------------------------------------------
void print_header(void)
{
        printf ("\n\n");
        printf ("\n+---------------------------------------------------+");
        printf ("\n| MSX-E370x MODBUS Sample                           |");
        printf ("\n+---------------------------------------------------+");
        printf ("\n| Press ESC to stop an acquisition                  |");
        printf ("\n+---------------------------------------------------+");
        printf ("\n\n");
}
//---------------------------------------------------------------------------------
static const char * typestr(uint32_t type)
{
	switch(type)
	{
	case 0:
		return "half-bridge";
	case 1:
		return "LVDT";
	case 2:
		return "Knaebel";
	case 3:
		return "half-bridge Mahr";
	case 4:
		return "LVDT Mahr";
	default:
		return "UNKNOWN";
	}
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerGetNbrOfType(struct modbus * modbus, uint32_t* NumberOfTransducerTypes)
{
	struct MX370x__TransducerGetNbrOfType_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__TransducerGetNbrOfType(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerGetNbrOfType") )
		return -1;

	*NumberOfTransducerTypes = Response.NumberOfTransducerTypes;
	printf("number of types defined in the transducer database: %d\n",Response.NumberOfTransducerTypes);
	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerGetNbrOfTypeEx(struct modbus * modbus, uint32_t* NumberOfTransducerTypes)
{
	struct MX370x__TransducerGetNbrOfTypeEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__TransducerGetNbrOfTypeEx(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerGetNbrOfTypeEx") )
		return -1;

	*NumberOfTransducerTypes = Response.NumberOfTransducerTypes;
	printf("number of types defined in the transducer database: %d\n",Response.NumberOfTransducerTypes);
	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__GetTransducerDatabaseCursor(struct modbus * modbus, uint32_t* TransducerDatabaseCursor)
{
	struct MX370x__GetTransducerDatabaseCursorEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__GetTransducerDatabaseCursorEx(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__GetTransducerDatabaseCursorEx") )
		return -1;

	*TransducerDatabaseCursor = Response.TransducerDatabaseCursor;
	printf("Current transducer database cursor: %d\n",Response.TransducerDatabaseCursor);
	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__GetTransducerDatabaseCursorEx(struct modbus * modbus, uint32_t* TransducerDatabaseCursor)
{
	struct MX370x__GetTransducerDatabaseCursorEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__GetTransducerDatabaseCursorEx(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__GetTransducerDatabaseCursorEx") )
		return -1;

	*TransducerDatabaseCursor = Response.TransducerDatabaseCursor;
	printf("Current transducer database cursor: %d\n",Response.TransducerDatabaseCursor);
	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerGetTypeInformation(struct modbus * modbus, struct MX370x__TransducerGetTypeInformation_parameters_t* TransducerGetTypeInformation)
{
	struct MX370x__TransducerGetTypeInformation_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__TransducerGetTypeInformation(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerGetTypeInformation") )
		return -1;

	memcpy(TransducerGetTypeInformation, &Response, sizeof(struct MX370x__TransducerGetTypeInformation_parameters_t));

	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerGetTypeInformationEx(struct modbus * modbus, struct MX370x__TransducerGetTypeInformationEx_parameters_t* TransducerGetTypeInformation)
{
	struct MX370x__TransducerGetTypeInformationEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__TransducerGetTypeInformationEx(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerGetTypeInformationEx") )
		return -1;

	memcpy(TransducerGetTypeInformation, &Response, sizeof(struct MX370x__TransducerGetTypeInformationEx_parameters_t));

	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__SetTransducerDatabaseCursor(struct modbus * modbus, uint32_t new)
{
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	ret = modbus_call_MX370x__SetTransducerDatabaseCursor(modbus, new, &CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__SetTransducerDatabaseCursor");
}
//---------------------------------------------------------------------------------
int sample_MX370x__SetTransducerDatabaseCursorEx(struct modbus * modbus, uint32_t new)
{
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	ret = modbus_call_MX370x__SetTransducerDatabaseCursorEx(modbus, new, &CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__SetTransducerDatabaseCursorEx");
}
//---------------------------------------------------------------------------------
/** prints the content of the transducer database on STDOUT */
int dump_transducer_database(struct modbus * modbus)
{
	uint32_t NumberOfTransducerTypes = 0;

	printf ("\n-----------------------------------------------------\n");
	printf ("Printing the content of the transducer database\n");
	printf ("-----------------------------------------------------\n");

	assert( sample_MX370x__TransducerGetNbrOfTypeEx(modbus, &NumberOfTransducerTypes) == 0 );

	/* go through the transducer database */
	{
		int i;
		for(i=0; i<NumberOfTransducerTypes; i++)
		{
			uint32_t TransducerDatabaseCursor;
			struct MX370x__TransducerGetTypeInformationEx_parameters_t TransducerGetTypeInformation;

			assert( sample_MX370x__SetTransducerDatabaseCursorEx(modbus, i) == 0);
			printf("\n");
			assert( sample_MX370x__GetTransducerDatabaseCursorEx(modbus, &TransducerDatabaseCursor) == 0);
			assert( i == TransducerDatabaseCursor );
			assert( sample_MX370x__TransducerGetTypeInformationEx(modbus, &TransducerGetTypeInformation) == 0 );

			printf("Selection index: %u\n", TransducerGetTypeInformation.SelectionIndex);
			printf("Name: %s\n", TransducerGetTypeInformation.Name);
			printf("Calibrated ? %s\n", ( TransducerGetTypeInformation.CalibrationStatus ? "yes" : "no" ) );
			printf("Type: %s\n", typestr(TransducerGetTypeInformation.Type));
			printf("Frequency: %u Hz\n", TransducerGetTypeInformation.Frequency);
			printf("Impedance: %u Ohm\n", TransducerGetTypeInformation.Impedance);
			printf("Nominal voltage: %f Vrms\n", TransducerGetTypeInformation.NominalVoltage);
			printf("Sensibility: %f mv/V/mm\n", TransducerGetTypeInformation.Sensibility);
			printf("Range: +/- %f mm\n", TransducerGetTypeInformation.Range);
			printf("\n");
		}
	}

	return 0;
}


//---------------------------------------------------------------------------------
int main(int argc, char** argv)
{
	struct modbus modbus;
	uint32_t NumberOfChannels;
	char address[100];

	if (argc != 1)
	{
		printf("%s does not take a parameter\n",argv[0]);
		exit(1);
	}

	print_header();
	modbus_init(&modbus);

	/* ask for network address */
	while( addidata_network_ask("module IP address?>", address) )
		printf("error while parsing IP address (CTRL+C to end)\n");

	switch( modbus_connect(&modbus, address, SOCK_DGRAM, 0) )
	{
	case 0: /* success */
		break;
	case -1: /* system error */
		addidata_network_perror("modbus_connect");
		exit(1);
	case -2: /* parameter is incorrect */
		printf("modbus_connect: incorrect parameter\n");
		exit(1);
	case -3: /* incorrect address */
		printf("incorrect address\n");
		exit(1);
	}


	printf ("-----------------------------------------------------\n");
	printf ("Calling some common functions\n");
	printf ("-----------------------------------------------------\n");

	assert( sample_MXCommon__GetModuleTypeEx(&modbus) == 0);
	assert( sample_MXCommon__GetTimeEx(&modbus) == 0);

	printf ("-----------------------------------------------------\n");
	printf ("Calling system specific functions\n");
	printf ("-----------------------------------------------------\n");

	assert( sample_MX370x__getNumberOfChannelsEx(&modbus, &NumberOfChannels) == 0);
	assert( dump_transducer_database(&modbus) == 0);

	assert( sample_MXCommon__SetHardwareTriggerFilterTimeEx(&modbus, 1) == 0);

	{
		int AcquisitionType;
		/* set formats for output */
		struct msxe370x_format format;

		format.counter = "counter:%d ";
		format.dataheader = "%u:";
		format.hwtrigger = "HW trigger info: 0x%X\t";
		format.digital = "%u";
		format.analog = "%f";
		format.datasep = " ";
		format.recordterm = "\n";

		/* general questions */
		AcquisitionType = addidata_console_get_unsigned_integer("Type of acquisition [0: autorefresh 1:sequence] setOffset [2]", 0,2);

		printf("\n");

		switch(AcquisitionType)
		{
		case 0: /* autorefresh */
		{
			int NumberOfAcquisition;
			int UseModbusFunction=0;
			struct msxe370x_autorefresh_parameters Params;

			NumberOfAcquisition = addidata_console_get_unsigned_integer("Number of acquisition ", 1,1000000);
			Params.TransducerSelection = addidata_console_get_unsigned_integer("Transducer selection index", 0,255);

			msxe370x_ask_autorefresh_parameters(&Params, NumberOfChannels);
			msxe370x_ask_autorefresh_format(&Params);

			printf("\n0\tFetch data with MODBUS\n");
			printf("1\tGet data via the dataserver\n");
			UseModbusFunction = !addidata_console_get_yn("Choice?");

			assert( sample_MX370x__TransducerInitAndStartAutoRefreshEx(&modbus, &Params, NumberOfChannels) == 0);

			if(UseModbusFunction)
			{
				int i;
				key_thread_handle_t kt;
				unsigned int stop = 0;
				wait_key_input_thread_parameters_t ktparams;
				ktparams.character = 27; // ESC
				ktparams.flag = &stop;

				kt = addidata_start_key_thread(&ktparams);
				{
					for(i=0;i<NumberOfAcquisition;i++)
					{
						assert( sample_MX370x__TransducerGetAutoRefreshValuesEx(&modbus, &format, &Params) == 0);
						if(stop)
						{
							break;
						}
					}
				}
				addidata_stop_key_thread(kt);
			}
			else
			{
				int fd;
				uint32_t* data;
				unsigned int count = msxe370x_autorefresh_compute_fields(&Params);
				data = malloc(count*4);
				if(!data)
				{
					perror("malloc");
					exit(1);
				}

				/* connect to data server */
				switch(addidata_dataclient_tcp_connect(address, 0, &fd))
				{
					case 0: /* OK */
						break;
					case -1:
						addidata_network_perror("socket");
						exit(1);
					case -2:
						addidata_network_perror("connect");
						exit(1);
					case -3:
						printf("error while parsing address\n");
						exit(1);
					case -4:
						addidata_network_perror("WSACleanup() failed");
						exit(1);
				}
				/* set timeout */
				{
					struct timeval timeout = MODBUS_NETWORK_TIMEOUT;
					if ( addidata_network_set_socket_timeout(fd, timeout) )
					{
						addidata_network_perror("setting timeout");
						exit(1);
					}
				}
				{
					int i;
					key_thread_handle_t kt;
					unsigned int stop = 0;
					wait_key_input_thread_parameters_t ktparams;
					ktparams.character = 27; // ESC
					ktparams.flag = &stop;
					kt = addidata_start_key_thread(&ktparams);
					for(i=0;i<NumberOfAcquisition;i++)
					{
						if( -1 == addidata_dataclient_recvall(fd, data, count) )
						{
							addidata_network_perror("addidata_dataclient_recvall");
							assert(sample_MX370x__TransducerStopAndReleaseAutoRefreshEx(&modbus) == 0);
							exit(1);
						}
						msxe370x_autorefresh_printf(&Params, data, count, &format);
						if(stop)
						{
							break;
						}
					}
					addidata_stop_key_thread(kt);
				}
				addidata_dataclient_disconnect(fd);
			}
			assert( sample_MX370x__TransducerStopAndReleaseAutoRefreshEx(&modbus) == 0);
			break;
		}
		case 1: /* sequence */
			{
				struct msxe370x_sequence_parameters Params;
				Params.TransducerSelection = addidata_console_get_unsigned_integer("Transducer selection index", 0,255);
				msxe370x_ask_sequence_parameters(&Params, NumberOfChannels);
				msxe370x_ask_sequence_format(&Params);

				/* fetch start acquisition, fetch data */
				{
					int fd;
					uint32_t* data;
					unsigned int count = msxe370x_sequence_compute_fields(&Params);
					printf("expected %d fields\n",count);
					data = malloc(count*4);
					if(!data)
					{
						perror("malloc");
						exit(1);
					}

					/* connect to data server */
					switch(addidata_dataclient_tcp_connect(address, 0, &fd))
					{
						case 0: /* OK */
							break;
						case -1:
							addidata_network_perror("socket");
							exit(1);
						case -2:
							addidata_network_perror("connect");
							exit(1);
						case -3:
							printf("error while parsing address\n");
							exit(1);
						case -4:
							addidata_network_perror("WSACleanup() failed");
							exit(1);
					}
					/* set timeout */
					{
						struct timeval timeout = MODBUS_NETWORK_TIMEOUT;
						if ( addidata_network_set_socket_timeout(fd, timeout) )
						{
							addidata_network_perror("setting timeout");
							exit(1);
						}
					}
					/* start acquisition */
					assert(sample_MX370x__TransducerInitAndStartSequenceEx(&modbus, &Params, NumberOfChannels) == 0);

					{
						key_thread_handle_t kt;
						unsigned int i = 0;
						unsigned int stop = 0;
						wait_key_input_thread_parameters_t ktparams;
						ktparams.character = 27; // ESC
						ktparams.flag = &stop;
						kt = addidata_start_key_thread(&ktparams);
						while(1)
						{
							if( -1 == addidata_dataclient_recvall(fd, data, count) )
							{
								addidata_network_perror("fetch");
								assert(sample_MX370x__TransducerStopAndReleaseSequenceEx(&modbus) == 0);
								exit(1);
							}
							msxe370x_sequence_printf(&Params, data, count, &format);
							i++;
							if( (Params.NbrOfSequence!=0) && (i==Params.NbrOfSequence) )
								break;
							if(stop)
							{
								break;
							}

						}
						addidata_stop_key_thread(kt);
					}
					addidata_dataclient_disconnect(fd);
				}

				/* stop acquisition */

				assert(sample_MX370x__TransducerStopAndReleaseSequenceEx(&modbus) == 0);
				break;
			}

		case 2:
		{
			assert(sample_MX370x__TransducerSetOffsetEx(&modbus) == 0);
			break;
		}
		}
		printf("\npress a key to end this sample\n");
		fflush(stdout);
		addidata_console_wait_for_input();
		return 0;
	}
}
