/** @file AutoRefresh.c
* AutoRefresh : MX370x : Auto Refresh acquisition
* 
*
* @author C. Guinot
* @date 27.03.2006
* 
* @version  $LastChangedRevision$
* @version $LastChangedDate$
* 
* 
* @par LICENCE
* @verbatim
*  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
*        
*        ADDI-DATA GmbH 
*        Airport Boulevard B210
*        D-77836 Rheinmnster Germany 
*        Tel: +49(0)7229/1847-0 
*        Fax: +49(0)7229/1847-200
*        http://www.addi-data-com 
*        info@addi-data.com 
*        
*
* @endverbatim   
*
* @par UPDATES                                                                                                          
* @verbatim                                                                                                             
*  Date      : 01.06.06                                                                                                 
*  Author        : C. Guinot                                                                                            
*  Description   : Remove the "_" character in the SOAP parameter name
*                                                                                                                       
*  Date      : 14.08.06                                                                                                 
*  Author        : C. Guinot                                                                                            
*  Description   : Add the data format management
*
*  Date      : 29.10.08
*  Author        : S. Weber
*  Description   : Add the read SOAP transducer data management
*
* @endverbatim     
*
*
*/ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>


#include "..\..\..\..\Interface_Library\MX370x.nsmap"
#include "..\util\socket.h"
#include "..\util\utility.h"



/*******************************************************************************************************/
/*                                       UTILS Functions                                               */               
/*******************************************************************************************************/



INT i_UTILS_GetTheNumberOfChannels (PCHAR pc_ModuleType, DWORD * pdw_NbrOfChannels)
{
	DWORD dw_Cpt = 0;
	CHAR pc_NbrOfChannels [5];
	DWORD dw_NbrOfChannelIndex = 0;

	/* check presence of the substring "UNKNOWN" indicating a not yet configured module */
	if (strstr(pc_ModuleType,"UNKNOWN"))
	{
		fprintf(stderr,"! Remote module not configured\n");
		return -1;
	}

	/* If DIO system, the pattern is not the same */
	if (strstr(pc_ModuleType, "DIO"))
	{
		/* module is like: MSX-E3701-DIO-HB-XX or MSX-E3701-DIO-LVDT-XX*/	
		/* get a pointer to HB or LVDT */
		PCHAR beg = strstr(pc_ModuleType, "LVDT");
		if ( ! beg )
			beg = strstr(pc_ModuleType, "HB");
		if ( ! beg )
		{
			fprintf(stderr,"! Unknown system: %s \n", pc_ModuleType);
			return -1;
		}

		/* loop until we find the first "-" caracter */
		for (dw_Cpt = 0; dw_Cpt < strlen(beg);dw_Cpt++)
		{
			if (beg [dw_Cpt] == '-') 
			{
				dw_Cpt++;
				break;
			}
		}

		/* now we have the number of channels */
		for ( ; dw_Cpt < strlen(beg); dw_Cpt++)
		{
			if ( (beg [dw_Cpt] < '0') ||(beg [dw_Cpt] > '9') )
			{
				break;
			}
			pc_NbrOfChannels[dw_NbrOfChannelIndex] = beg[dw_Cpt];
			dw_NbrOfChannelIndex++;
		}

		/* check */
		if ( dw_NbrOfChannelIndex == 0 )
			return -1;

		/* convert and return */
		pc_NbrOfChannels[dw_NbrOfChannelIndex] = '\0';
		*pdw_NbrOfChannels = atol (pc_NbrOfChannels);
		return 0;
	}

	/* Normal systems */
	for (dw_Cpt = 0; dw_Cpt < strlen(pc_ModuleType);dw_Cpt++)
	{
		if (pc_ModuleType [dw_Cpt] == '-') 
		{
			dw_Cpt++;
			break;
		}
	}

	if (dw_Cpt < strlen(pc_ModuleType))
	{
		for (dw_NbrOfChannelIndex = 0; dw_NbrOfChannelIndex < (strlen(pc_ModuleType) - dw_Cpt ); dw_NbrOfChannelIndex ++)
		{
			if (pc_ModuleType [dw_Cpt + dw_NbrOfChannelIndex] == '-')
				break;
			else
				pc_NbrOfChannels [dw_NbrOfChannelIndex] = pc_ModuleType [dw_Cpt + dw_NbrOfChannelIndex];

		}
		pc_NbrOfChannels[dw_NbrOfChannelIndex] = '\0';

		if (dw_NbrOfChannelIndex < (strlen(pc_ModuleType) - dw_Cpt ))
			*pdw_NbrOfChannels = atol (pc_NbrOfChannels);
		else
			return -1;
	}
	else
		return -1;
	
	return 0;
}

/*******************************************************************************************************/
/*                                       Transducer Functions                                           */
/*******************************************************************************************************/


int i_DisplayAndSelectTransducerType(struct soap *ps_SoapEnv,PCHAR pc_SOAPIPAddress,DWORD * pdw_TransducerTypeSelection)
{
	struct MX370x__unsignedlongResponse s_Response;
	struct MX370x__TransducerGetTypeInformationResponse TypeInfoResponse;

	INT i_Error = 0;
	DWORD dw_Cpt = 0;
	DWORD pdw_SelectionValueArray[100];
	DWORD dw_Choice = 0;
	DWORD dw_Dummy = 0;
  
	/* Get the number of transducer type */
	i_Error = soap_call_MX370x__TransducerGetNbrOfType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_Response);
	if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_Response.iReturnValue, 0, "soap_call_MX370x__TransducerGetNbrOfType", 0))
		return -1;
	else
	{
		printf("\nNumber of transducer types: %d\n", s_Response.ulValue);

		for (dw_Cpt = 0; dw_Cpt < s_Response.ulValue; dw_Cpt ++)
		{
			/* Get the transducer information */
			i_Error = soap_call_MX370x__TransducerGetTypeInformation (ps_SoapEnv,pc_SOAPIPAddress, SOAP_ACTION, dw_Cpt, &TypeInfoResponse);
			if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, TypeInfoResponse.iReturnValue, 0, "soap_call_MX370x__TransducerGetTypeInformation", 0))
				return -2;
			else
			{
				printf("\n");
				printf("Transducer Index: %d\n", dw_Cpt);
				printf("Selection Index: %d\n", TypeInfoResponse.ulTransducerSelectionIndex);

				pdw_SelectionValueArray[dw_Cpt] = TypeInfoResponse.ulTransducerSelectionIndex;
				printf("Name: %s\n", TypeInfoResponse.pcName);
				printf("Calibration status: %d\n", TypeInfoResponse.ulCalibrationStatus);
				printf("Type value: %d\n", TypeInfoResponse.ulType);
				printf("Frequency: %d Hz\n", TypeInfoResponse.ulFrequency);
				printf("Impedance: %d Ohm\n", TypeInfoResponse.ulImpedance);
				printf("Nominal Voltage: %lf V\n", TypeInfoResponse.dVeff);
				printf("Sensibility: %lf mV/V/mm\n", TypeInfoResponse.dSensibility);
				printf("Range: +/- %lf mm\n", TypeInfoResponse.dRange);
			}
		}

		do 
		{
			printf("\nEnter the index of the transducer type you want ot use (0 to %d): ", (s_Response.ulValue - 1));
			scanf("%d",&dw_Choice);
		}
		while (dw_Choice >= s_Response.ulValue);
		*pdw_TransducerTypeSelection = pdw_SelectionValueArray[dw_Choice];
	}
	return 0;
}

void GetAutoRefreshParameters(DWORD dw_NbrOfChannels, 
                              DWORD *pdw_ChannelMask, 
                              DWORD *pdw_AverageMode, 
                              DWORD *pdw_AverageValue, 
                              DWORD *pdw_DivisionFactor,
                              DWORD *pdw_TriggerAction, 
                              DWORD *pdw_HardwareTriggerCount, 
                              DWORD *pdw_HardwareTriggerFilterTime, 
                              DWORD *pdw_ByTriggerNbrOfSeqToAcquire,
                              DWORD *pdw_DataFormat,
							  BYTE  *pdw_AcquisitionMode)
{
	DWORD dw_Choice = 0;


	*pdw_ChannelMask                = 0;
	*pdw_AverageMode                = 0;
	*pdw_AverageValue               = 0;
	*pdw_DivisionFactor             = 0;
	*pdw_TriggerAction              = 0;
	*pdw_HardwareTriggerCount       = 0;
	*pdw_HardwareTriggerFilterTime  = 0;
	*pdw_ByTriggerNbrOfSeqToAcquire = 0;
	*pdw_DataFormat                 = 0;


	do
	{
		printf("\nEnter the channel mask (in Hex)"); 
		printf("\ne.g: 1=CH1, 2=CH2, 3=CH1 and CH2, 4=CH3, 7=CH1 and CH2 and CH3: ");
		scanf("%X", pdw_ChannelMask);
	}
	while ((*pdw_ChannelMask == 0) || (*pdw_ChannelMask > (pow(2,dw_NbrOfChannels) - 1)));

	printf("\nDo You want to activate the average mode (0 or 1)? ");
	scanf("%d",&dw_Choice);

	if (dw_Choice == 1)
	{
		do
		{
			printf("\n1. Average mode per sequence\n");
			printf("2. Average mode per channel\n");
			printf("Select the Average mode do You want to use: ");
			scanf("%d", pdw_AverageMode);
		}
		while ((*pdw_AverageMode != 1) &&(*pdw_AverageMode != 2));

		do
		{
			printf("\nEnter the average value (2 to 255): ");
			scanf("%d", pdw_AverageValue);
		}
		while ((*pdw_AverageValue < 2) || (*pdw_AverageValue > 255));
	}

	do
	{
		printf("\nEnter the division factor (5 to 255): ");
		scanf("%d", pdw_DivisionFactor);
	}
	while ((*pdw_DivisionFactor < 5) || (*pdw_DivisionFactor > 255));

// trigger start
	printf("\nDo You want to use the hardware trigger to start the acquisition (0 or 1)? ");
	scanf("%d", &dw_Choice);

	if (dw_Choice == 1)
	{
		do
		{
			printf("\n1. One shot trigger\n");
			printf("2. Sequence trigger\n");
			printf("Select the trigger mode do You want to use: ");
			scanf("%d", pdw_TriggerAction);
		}
		while ((*pdw_TriggerAction != 1) && (*pdw_TriggerAction != 2));

		do 
		{
			printf("\n1. rising edge\n");
			printf("2. falling edge\n");
			printf("3. both edges\n");
			printf("Select the active edge: ");
			scanf ("%d", &dw_Choice);
		}
		while ((dw_Choice < 1) || (dw_Choice > 3));

		*pdw_TriggerAction = *pdw_TriggerAction | (dw_Choice << 6);

		do
		{
			printf("\nEnter the number of edges before the trigger action occure (1 to 65535): ");
			scanf("%d",pdw_HardwareTriggerCount);
		}
		while ((*pdw_HardwareTriggerCount == 0) || (*pdw_HardwareTriggerCount > 65535));

		do
		{
			printf("\nEnter the filter time (multiplier from 250 ns step) for the input trigger (0 = disable, max : 65535 -> 16,383750 ms): ");
			scanf("%d",pdw_HardwareTriggerFilterTime);
		}
		while (*pdw_HardwareTriggerFilterTime > 65535);
	}
	else
	{
		printf("\nDo You want to use the synchronisation trigger to start the acquisition(0 or 1)? ");
		scanf("%d", &dw_Choice);

		if (dw_Choice == 1)
		{
			do
			{
				printf("\n1. One shot trigger\n");
				printf("2. Sequence trigger\n");
				printf("Select the trigger mode do You want to use: ");
				scanf("%d", &dw_Choice);
			}
			while ((dw_Choice != 1) && (dw_Choice != 2));

		    *pdw_TriggerAction = dw_Choice << 8;
		}
	}

	if (((*pdw_TriggerAction & 2) == 0x2) || ((*pdw_TriggerAction & 0x200) == 0x200))
	{
		do
		{
			printf("\nEnter the number of sequence to acquire by each trigger: ");
			scanf("%d", pdw_ByTriggerNbrOfSeqToAcquire);
		}
		while (*pdw_ByTriggerNbrOfSeqToAcquire == 0);
	}


//	Trigger Stop
	if ((*pdw_TriggerAction == 0) || ((*pdw_TriggerAction & 0x101) != 0))
	{
		printf("\nDo You want to use the hardware trigger to stop the acquisition (0 or 1)? ");
		scanf("%d", &dw_Choice);

		if (dw_Choice == 1)
		{
			*pdw_TriggerAction = *pdw_TriggerAction | (1 << 16); 

			do 
			{
				printf("\n1. rising edge\n");
				printf("2. falling edge\n");
				printf("3. both edges\n");
				printf("Select the active edge: ");
				scanf ("%d", &dw_Choice);
			}
			while ((dw_Choice < 1) || (dw_Choice > 3));

			*pdw_TriggerAction = *pdw_TriggerAction | (dw_Choice << 17);

			do
			{
				printf("\nDo you want to use the number of edge before the trigger occur (0 or 1): ");
				scanf("%d",&dw_Choice);
			}
			while (dw_Choice > 1);

			if (dw_Choice == 1)
			{
				*pdw_TriggerAction = *pdw_TriggerAction | (1 << 19);

				if (*pdw_HardwareTriggerCount == 0)
				{
					do
					{
						printf("\nEnter the number of edges before the trigger action occur (1 to 65535): ");
						scanf("%d",pdw_HardwareTriggerCount);
					}
					while ((*pdw_HardwareTriggerCount == 0) || (*pdw_HardwareTriggerCount > 65535));
				}
			}

		}
	}


	do
	{
		printf("\n0. Acquisition via the SOAP function\n");
		printf("1. Acquisition via the data server\n");
		printf("Select the acquisition mode You want to use: ");
		scanf("%d", pdw_AcquisitionMode);
	}
	while ((*pdw_AcquisitionMode != 0) && (*pdw_AcquisitionMode != 1));

	// Test if data server used
	if (*pdw_AcquisitionMode == 1)
	{
		do 
		{
			printf("\nDo You want to receive a time stamp (0: No, 1: Yes) ?");
			scanf ("%d",&dw_Choice);
		}
		while (dw_Choice > 1);

		*pdw_DataFormat = dw_Choice;

		do 
		{
			printf("\nDo You want to receive the value in digital form (tape 0) or in analog form mm (type 1) ?");
			scanf ("%d",&dw_Choice);
		}
		while (dw_Choice > 1);

		*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 1);
		
		do 
		{
			printf("\nDo You want to invert the value sign (0: No, 1: Yes) ?");
			scanf ("%d",&dw_Choice);
		}
		while (dw_Choice > 1);

		*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 2);
	}

}


typedef struct
      {
      DWORD       dw_DataFormat;
	  DWORD       dw_ChannelMask;
      DWORD       dw_NumberOfChannelsToAcquire;
      BYTE         b_Exit;
      SOCKET       s_ClientSocket;
	  FILE         *fp;
	  struct soap  *ps_SoapEnv;
	  char         *pc_SOAPIPAddress;
      }str_ThreadInfo;



#pragma pack(1)
typedef union
      {
      struct
	    {
	    DWORD dw_TimeStamp[2];
	    DWORD dw_Counter;
	    DWORD dw_BinaryValue[16];
	    }s_WichtTimeStampBinaryData;

	 struct
	    {
	    DWORD dw_TimeStamp[2];
	    DWORD dw_Counter;
	    FLOAT  f_RealValue[16];
	    }s_WichtTimeStampRealData;

	 struct
	    {
	    DWORD dw_Counter;
	    DWORD dw_BinaryValue[16];
	    }s_WichtoutTimeStampBinaryData;


	 struct
	    {
	    DWORD dw_Counter;
	    FLOAT  f_RealValue[16];
	    }s_WichtoutTimeStampRealData;
	 }str_AutoRefreshData;
#pragma pack()


DWORD WINAPI dw_DataServerClientThread (LPVOID lpParameter)
        {
        str_ThreadInfo * ps_ThreadInfo = (str_ThreadInfo *) lpParameter;
		INT i_ReturnValue              = 0;
    	DWORD dw_Cpt                   = 0;
    	DWORD dw_RequestValue          = 0;
    	DWORD dw_NbrOfReceiveValues    = 0;
    	DWORD dw_NbrOfValuesToReceive  = 0;
		CHAR *pb_Buffer                = NULL;
		str_AutoRefreshData s_AutoRefreshData;

        do
        {
		    /* Test if the data format time stamp is activated */
			if (ps_ThreadInfo->dw_DataFormat & 1)
			{
                dw_NbrOfValuesToReceive = (ps_ThreadInfo->dw_NumberOfChannelsToAcquire + 3) * sizeof (DWORD);
			}
			else
			{
                dw_NbrOfValuesToReceive = (ps_ThreadInfo->dw_NumberOfChannelsToAcquire + 1) * sizeof (DWORD);
			}
				
			dw_RequestValue = dw_NbrOfValuesToReceive;
			dw_NbrOfReceiveValues = 0;
			pb_Buffer = (CHAR *)&s_AutoRefreshData.s_WichtoutTimeStampBinaryData;

			do
			{
				i_ReturnValue = recv (ps_ThreadInfo->s_ClientSocket, (PCHAR)&pb_Buffer[dw_NbrOfReceiveValues] , dw_RequestValue, 0); 

				if (i_ReturnValue != SOCKET_ERROR)
				{
					dw_NbrOfReceiveValues = dw_NbrOfReceiveValues + i_ReturnValue;
					dw_RequestValue = dw_RequestValue - i_ReturnValue;
				}
			}
			while ((dw_NbrOfReceiveValues < dw_NbrOfValuesToReceive) && (i_ReturnValue != SOCKET_ERROR));

            printf("\n");

			if (i_ReturnValue != SOCKET_ERROR)
			{

		        /* Test if the data format time stamp is activated */
			    if (ps_ThreadInfo->dw_DataFormat & 1)
				{					
					if (ps_ThreadInfo->fp != NULL)
					{
						fprintf (ps_ThreadInfo->fp,"%s;", DisplayTimeStampCSV (s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_TimeStamp[0],s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_TimeStamp[1]));
					}
					else
						DisplayTimeStamp (s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_TimeStamp[0],s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_TimeStamp[1]);

				    printf("Counter: %lu", s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_Counter);
					if (ps_ThreadInfo->fp != NULL)
						fprintf(ps_ThreadInfo->fp, "Counter: %lu;", s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_Counter);

					/* Test if receive analog value */
					if (ps_ThreadInfo->dw_DataFormat & 2)
					{
						for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
						{ 
							printf(" %lu:%fmm",dw_Cpt+1,s_AutoRefreshData.s_WichtTimeStampRealData.f_RealValue [dw_Cpt]);
							if (ps_ThreadInfo->fp != NULL)
								fprintf(ps_ThreadInfo->fp, "%lu;%f;mm;",dw_Cpt+1,s_AutoRefreshData.s_WichtTimeStampRealData.f_RealValue [dw_Cpt]);
						}
					}
					else
					{
						for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
						{ 
							printf(" %lu:%lX",dw_Cpt+1,s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_BinaryValue[dw_Cpt]);
							if (ps_ThreadInfo->fp != NULL)
								fprintf(ps_ThreadInfo->fp, "%lu;%lX;",dw_Cpt+1,s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_BinaryValue[dw_Cpt]);
						}
					}
				}
				else
				{
				    printf("Counter: %lu", s_AutoRefreshData.s_WichtoutTimeStampBinaryData.dw_Counter);
					if (ps_ThreadInfo->fp != NULL)
						fprintf(ps_ThreadInfo->fp, "Counter: %lu;", s_AutoRefreshData.s_WichtoutTimeStampBinaryData.dw_Counter);

					/* Test if receive analog value */
					if (ps_ThreadInfo->dw_DataFormat & 2)
					{
						for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
						{ 
							printf(" %lu:%fmm",dw_Cpt+1,s_AutoRefreshData.s_WichtoutTimeStampRealData.f_RealValue [dw_Cpt]);
							if (ps_ThreadInfo->fp != NULL)
								fprintf(ps_ThreadInfo->fp, "%lu;%f;mm;",dw_Cpt+1,s_AutoRefreshData.s_WichtoutTimeStampRealData.f_RealValue [dw_Cpt]);
						}
					}
					else
					{
						for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
						{ 
							printf(" %lu:%lX",dw_Cpt+1,s_AutoRefreshData.s_WichtoutTimeStampBinaryData.dw_BinaryValue [dw_Cpt]);
							if (ps_ThreadInfo->fp != NULL)
								fprintf(ps_ThreadInfo->fp, "%lu;%lX;",dw_Cpt+1,s_AutoRefreshData.s_WichtoutTimeStampBinaryData.dw_BinaryValue [dw_Cpt]);
						}
					}
				}
				if (ps_ThreadInfo->fp != NULL)
					fprintf(ps_ThreadInfo->fp,"\n");
			}
			else
			{
				/*****************************/
				/* Abnormal error, print it. */
				/*****************************/		
				if (ps_ThreadInfo->b_Exit != 1)
					printf("Socket Error\n");

        		ps_ThreadInfo->b_Exit = 1;
            }

        } 
        while (ps_ThreadInfo->b_Exit == 0);

        /*********************
        * Return from thread 
        **********************/
        return (0);

        }


DWORD WINAPI dw_SOAPThread (LPVOID lpParameter)
{
	int                                        Ret;
	int                                        Cpt;
	struct MX370x__unsignedlong17ArrayResponse Response;
    str_ThreadInfo * ps_ThreadInfo = (str_ThreadInfo *) lpParameter;

    do
    {
		printf("\n");

		// Get the auto refresh transducer values
		Ret = soap_call_MX370x__TransducerGetAutoRefreshValues (ps_ThreadInfo->ps_SoapEnv, ps_ThreadInfo->pc_SOAPIPAddress, SOAP_ACTION, NULL, &Response);

		if (!i_CheckAndPrintError (ps_ThreadInfo->ps_SoapEnv, ps_ThreadInfo->pc_SOAPIPAddress, SOAP_ACTION, Ret, Response.iReturnValue, 0, "soap_call_MX370x__TransducerGetAutoRefreshValues", 1))
		{
			printf("Counter: %lu", Response.ulValue[0]);
			if (ps_ThreadInfo->fp != NULL)
				fprintf(ps_ThreadInfo->fp, "Counter: %lu;", Response.ulValue[0]);

			for (Cpt = 0; Cpt < 16; Cpt ++)
			{
				if (((ps_ThreadInfo->dw_ChannelMask >> Cpt) & 1) == 1)
				{
					printf(" %lu:%lX",Cpt+1,Response.ulValue[Cpt + 1]);
					if (ps_ThreadInfo->fp != NULL)
						fprintf(ps_ThreadInfo->fp, "Channel %lu;%lu;",Cpt+1,Response.ulValue[Cpt + 1]);
				}
			}

			if (ps_ThreadInfo->fp != NULL)
				fprintf(ps_ThreadInfo->fp,"\n");

		}
		else
		{
			ps_ThreadInfo->b_Exit = 1;
		}

		/* 
		 * gSOAP doesn't release automatically the memory that 
		 * it allocates to deserialize data, that's why both 
		 * following functions have to be used to release the 
		 * allocated memory.
		 */
		soap_destroy(ps_ThreadInfo->ps_SoapEnv);
		soap_end(ps_ThreadInfo->ps_SoapEnv);

	} 
	while (ps_ThreadInfo->b_Exit == 0);

	return (0);
}



void main(void)
{
	CHAR              pc_IPAddressTemp [50];
	CHAR              pc_IPAddress     [50];
	CHAR              pc_SOAPIPAddress [50];
	FILE              *fp                           = NULL;
	BYTE               b_DataSocketOpen             = 0;
	DWORD             dw_SaveValues                 = 0;
	DWORD             dw_AcquisitionMode            = 0;
	INT                i_key                        = 0;
	INT                i_Error                      = 0;
	INT                i_ReturnValue                = 0;
	HANDLE             h_ThreadHandle               = NULL;
	SOCKET             s_ClientSocket;
	DWORD             dw_Cpt                        = 0;
	DWORD             dw_Dummy                      = 0;
	DWORD             dw_ThreadId                   = 0;
	DWORD             dw_NbrOfChannels              = 0;
	xsd__unsignedLong dw_DataFormat                 = 0;
	xsd__unsignedLong dw_ChannelMask                = 0;
	xsd__unsignedLong dw_AverageMode                = 0;
	xsd__unsignedLong dw_AverageValue               = 0;
	xsd__unsignedLong dw_DivisionFactor             = 0;
	xsd__unsignedLong dw_TriggerAction              = 0; 
	xsd__unsignedLong dw_ReservedParameter2         = 0;
	xsd__unsignedLong dw_ReservedParameter3         = 0;
	xsd__unsignedLong dw_ReservedParameter4         = 0;
	xsd__unsignedLong dw_HardwareTriggerCount       = 0;
	xsd__unsignedLong dw_TransducerTypeSelection    = 0;
	xsd__unsignedLong dw_HardwareTriggerFilterTime  = 0;
	xsd__unsignedLong dw_ByTriggerNbrOfSeqToAcquire = 0;
	struct soap      *ps_SoapEnv                    = NULL;
	struct MXCommon__ByteArrayResponse  s_ByteArrayResponse;
	struct MX370x__Response             s_Response;
    str_ThreadInfo   s_ThreadInfo;

	memset (&s_ThreadInfo, 0, sizeof (s_ThreadInfo));

	ShowWindow(GetForegroundWindow (), SW_SHOWMAXIMIZED);

	/****************************/
	/* Print sample information */
	/****************************/
	printf("+---------------------------------------------------+\n");
	printf("| MX370x AutoRefresh                                |\n");
	printf("| MX370x Auto Refresh Test Program                  |\n");
	printf("+---------------------------------------------------+\n");
	printf("| <ESC> exit sample                                 |\n");
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv,pc_SOAPIPAddress , SOAP_ACTION, &dw_Dummy,&s_ByteArrayResponse);
		if (!i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_ByteArrayResponse.sResponse.iReturnValue, s_ByteArrayResponse.sResponse.syserrno, "soap_call_MXCommon__GetModuleType", 0))
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);
			
			if (i_UTILS_GetTheNumberOfChannels(s_ByteArrayResponse.sArray.__ptr, &dw_NbrOfChannels) == 0)
			{
				printf("\t=> This is a %d transducers module\n",dw_NbrOfChannels);
				/* Get Transducer type selection */
				if(i_DisplayAndSelectTransducerType(ps_SoapEnv, pc_SOAPIPAddress, (DWORD *)&dw_TransducerTypeSelection) == 0)
				{
					/* Get the auto refresh initialisation parameter */
					GetAutoRefreshParameters(dw_NbrOfChannels, 
					                         &dw_ChannelMask, 
								             &dw_AverageMode, 
								             &dw_AverageValue, 
								             &dw_DivisionFactor,
								             &dw_TriggerAction, 
								             &dw_HardwareTriggerCount, 
								             &dw_HardwareTriggerFilterTime, 
								             &dw_ByTriggerNbrOfSeqToAcquire,
								             &dw_DataFormat,
											 &dw_AcquisitionMode);

					do 
					{
						printf("\nSave values in a file (0: No, 1: Yes) ?");
						scanf ("%d",&dw_SaveValues);
					}
					while (dw_SaveValues > 1);

					/* Define the number of data to receive */
					for (dw_Cpt = 0; dw_Cpt < dw_NbrOfChannels; dw_Cpt ++)
					{
						if (dw_ChannelMask & (1<<dw_Cpt))
							s_ThreadInfo.dw_NumberOfChannelsToAcquire ++;
					}


					// Test if data server used
					if (dw_AcquisitionMode == 1)
					{
						if ((i_Error = i_Socket_ClientOpen(pc_IPAddress,8989,&s_ClientSocket)) != 0)
						{
							printf("i_Socket_ClientOpen Error\n");
						}
						else
						{
							b_DataSocketOpen = 1;
						}
					}


					if (dw_SaveValues && i_Error == 0)
					{
						if ((fp = fopen ("values.csv", "w+")) == NULL)
						{
							printf ("fopen error\n");
							i_Error = -1;
						}
					}
	

					// Test if no error occur
					if (i_Error == 0)
					{
						printf("\n Press ESC to exit the sample...");

						// Save the thread information's
						s_ThreadInfo.b_Exit           = 0;
						s_ThreadInfo.fp               = fp;
						s_ThreadInfo.dw_DataFormat    = dw_DataFormat;
						s_ThreadInfo.s_ClientSocket   = s_ClientSocket;
						s_ThreadInfo.dw_ChannelMask   = dw_ChannelMask;
						s_ThreadInfo.ps_SoapEnv       = ps_SoapEnv;
						s_ThreadInfo.pc_SOAPIPAddress = pc_SOAPIPAddress;

						// Test if data server used
						if (dw_AcquisitionMode == 1)
						{
							// Create the data server client thread
							h_ThreadHandle = CreateThread (NULL,
							 	        				   0,
							 	        				   dw_DataServerClientThread,
							 	        				   &s_ThreadInfo,
							 	        				   0,
							 	        				   &dw_ThreadId);

							Sleep (100);
						}
       

						/* Start the auto refresh */
						i_Error = soap_call_MX370x__TransducerInitAndStartAutoRefresh (ps_SoapEnv,
																					   pc_SOAPIPAddress,
																					   SOAP_ACTION,
																					   dw_TransducerTypeSelection,
																					   dw_ChannelMask,
																					   dw_AverageMode,
																					   dw_AverageValue,
																					   dw_DivisionFactor,
																					   dw_TriggerAction,
																					   dw_HardwareTriggerCount,
																					   dw_HardwareTriggerFilterTime,
																					   dw_ByTriggerNbrOfSeqToAcquire,
																					   dw_DataFormat,
																					   dw_ReservedParameter2,
																					   dw_ReservedParameter3,
																					   dw_ReservedParameter4,
																					   &s_Response);
						if (!i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_Response.iReturnValue, s_Response.syserrno, "soap_call_MX370x__TransducerInitAndStartAutoRefresh", 0))
						{
							/* 
							 * gSOAP doesn't release automatically the memory that 
							 * it allocates to deserialize data, that's why both 
							 * following functions have to be used to release the 
							 * allocated memory.
							 */
							soap_destroy(ps_SoapEnv);
							soap_end(ps_SoapEnv);

							// Test if SOAP used
							if (dw_AcquisitionMode == 0)
							{
								// Create the data server client thread
								h_ThreadHandle = CreateThread (NULL,
								 	        				   0,
								 	        				   dw_SOAPThread,
								 	        				   &s_ThreadInfo,
							 		        				   0,
							 			    				   &dw_ThreadId);

							}

		                    do
				            {

							    if (kbhit()) 
							    {
								    i_key = getch();
						        }
								else
							    {
								    Sleep (100);
						        }
		                    }
				            while ((i_key != 27) && (!s_ThreadInfo.b_Exit));


						    s_ThreadInfo.b_Exit = 1;


						    Sleep (100);
				
				            // Free the thread 
				            if (h_ThreadHandle != NULL)
					        {
						        // Close the thread handle 
					            CloseHandle (h_ThreadHandle);
				
							}

							i_Error = soap_call_MX370x__TransducerStopAndReleaseAutoRefresh(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_Response);
							i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_Response.iReturnValue, s_Response.syserrno, "soap_call_MX370x__TransducerStopAndReleaseAutoRefresh", 0);
						}
					}

					// Test if data server used and open
					if ((dw_AcquisitionMode == 1) && (b_DataSocketOpen == 1))
					{
						/* Close the socket connection */
						i_Socket_ClientClose (s_ClientSocket);
					}

					// Test if file open
					if (fp != NULL)
					{
						fclose (fp);
					}
				}
			}
			else
			{
				printf("find the number of channel error\n");
			}
		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

   
	/**********************************/
	/*Print End programme informations*/
	/**********************************/
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));

}
