/** @file Calibration.c
* Calibration : MX370x : Calibration sample
* 
*
* @author S. Weber
* @date 16.05.2006
* 
* @version  $LastChangedRevision$
* @version $LastChangedDate$
* 
* 
* @par LICENCE
* @verbatim
*  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
*        
*        ADDI-DATA GmbH 
*        Airport Boulevard B210
*        D-77836 Rheinmnster Germany 
*        Tel: +49(0)7229/1847-0 
*        Fax: +49(0)7229/1847-200
*        http://www.addi-data-com 
*        info@addi-data.com 
*        
*
* @endverbatim   
*
* @par UPDATES                                                                                                          
* @verbatim                                                                                                             
*  Date      : 01.06.06                                                                                                 
*  Author        : C. Guinot                                                                                            
*  Description   : Remove the "_" character in the SOAP parameter name
*                                                                                                                       
* @endverbatim     
*
*/ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>


#include "..\..\..\..\Interface_Library\MX370x.nsmap"
#include "..\UTIL\utility.h"
#include "..\UTIL\socket.h"


#define _TRANSDUCER_HB		0
#define _TRANSDUCER_LVDT	1


/********************************/
/* Console management functions */
/********************************/
int    i_X             = 0;
int    i_Y             = 0;
INT    i_ThreadStartY  = 0;
INT    i_ThreadCurentY = 0;

HANDLE h_ConsoleMutex  = NULL;



/*******************************************************************************************************/
/*                                       UTILS Functions                                               */               
/*******************************************************************************************************/


INT i_UTILS_GetModuleInformations (PCHAR pc_ModuleType, 
                                   DWORD * pdw_NbrOfChannels,
								   DWORD * pdw_Version)
{
	DWORD dw_Cpt = 0;
	CHAR  pc_CopyString [20];
	DWORD dw_CopyCounter = 0;

	/* check presence of the substring "UNKNOWN" indicating a not yet configured module */
	if (strstr(pc_ModuleType,"UNKNOWN"))
	{
		fprintf(stderr,"! Remote module not configured\n");
		return -1;
	}

	/* If DIO system, the pattern is not the same */
	if (strstr(pc_ModuleType, "DIO"))
	{
		/* module is like: MSX-E3701-DIO-HB-XX or MSX-E3701-DIO-LVDT-XX*/	
		/* get a pointer to HB or LVDT */
		PCHAR beg = strstr(pc_ModuleType, "LVDT");
		if ( ! beg )
			beg = strstr(pc_ModuleType, "HB");
		if ( ! beg )
		{
			fprintf(stderr,"! Unknown system: %s \n", pc_ModuleType);
			return -1;
		}

		/* loop until we find the first "-" caracter */
		for (dw_Cpt = 0; dw_Cpt < strlen(beg);dw_Cpt++)
		{
			if (beg [dw_Cpt] == '-') 
			{
				dw_Cpt++;
				break;
			}
		}

		/* now we have the number of channels */
		for ( ; dw_Cpt < strlen(beg); dw_Cpt++)
		{
			if ( (beg [dw_Cpt] < '0') ||(beg [dw_Cpt] > '9') )
			{
				break;
			}
			pc_CopyString[dw_CopyCounter] = beg[dw_Cpt];
			dw_CopyCounter++;
		}

		/* check */
		if ( dw_CopyCounter == 0 )
			return -1;

		/* convert  */
		pc_CopyString[dw_CopyCounter] = '\0';
		*pdw_NbrOfChannels = atol (pc_CopyString);

		/* now we can get the revision (directly after the "}_" string)*/
		beg = strstr(pc_ModuleType, "}_");
		if ( ! beg )
		{
			fprintf(stderr,"! Unknown system revision: %s \n", pc_ModuleType);
			return -1;
		}

		dw_CopyCounter = 0;
		for ( dw_Cpt = 2; dw_Cpt < strlen(beg); dw_Cpt++)
		{
			if ( (beg [dw_Cpt] < '0') ||(beg [dw_Cpt] > '9') )
			{
				break;
			}
			pc_CopyString[dw_CopyCounter] = beg[dw_Cpt];
			dw_CopyCounter++;
		}

		/* check */
		if ( dw_CopyCounter == 0 )
			return -1;

		/* convert and return */
		pc_CopyString[dw_CopyCounter] = '\0';
		*pdw_Version = atol (pc_CopyString);

		return 0;
	}


	/* Normal systems */
	for (dw_Cpt = 0; dw_Cpt < strlen(pc_ModuleType);dw_Cpt++)
	{
		if (pc_ModuleType [dw_Cpt] == '-') 
		{
			dw_Cpt++;
			break;
		}
	}

	if (dw_Cpt < strlen(pc_ModuleType))
	{
		for (dw_CopyCounter = 0; dw_CopyCounter < (strlen(pc_ModuleType) - dw_Cpt ); dw_CopyCounter ++)
		{
			if (pc_ModuleType [dw_Cpt + dw_CopyCounter] == '-')
				break;
			else
				pc_CopyString [dw_CopyCounter] = pc_ModuleType [dw_Cpt + dw_CopyCounter];

		}
		pc_CopyString[dw_CopyCounter] = '\0';

		if (dw_CopyCounter < (strlen(pc_ModuleType) - dw_Cpt ))
		{
			*pdw_NbrOfChannels = atol (pc_CopyString);


			for (dw_Cpt = 0; dw_Cpt < strlen(pc_ModuleType);dw_Cpt++)
			{
				if (pc_ModuleType [dw_Cpt] == '_') 
				{
					dw_Cpt++;
					break;
				}
			}

			if (dw_Cpt < strlen(pc_ModuleType))
			{
				for (dw_CopyCounter = 0; dw_CopyCounter < (strlen(pc_ModuleType) - dw_Cpt ); dw_CopyCounter ++)
				{
					if (pc_ModuleType [dw_Cpt + dw_CopyCounter] == ' ')
						break;
					else
						pc_CopyString [dw_CopyCounter] = pc_ModuleType [dw_Cpt + dw_CopyCounter];

				}
				
				pc_CopyString[dw_CopyCounter] = '\0';

				if (dw_CopyCounter < (strlen(pc_ModuleType) - dw_Cpt ))
				{
					*pdw_Version = atol (pc_CopyString);
				}
				else
				{
					return -1;
				}
			}
			else
				return -1;

		}
		else
		{
			return -1;
		}
	}
	else
		return -1;
	
	return 0;
}


int i_DisplayAndSelectTransducerType(struct soap *ps_SoapEnv,
                                     PCHAR     pc_SOAPIPAddress,
									 DWORD  * pdw_TransducerTypeSelection, 
									 double *  pd_Range,
									 DWORD *pdw_LVDTOrHB)
{
	struct MX370x__unsignedlongResponse Response;
	struct MX370x__TransducerGetTypeInformationResponse TypeInfoResponse;

	INT i_Error = 0;
	DWORD dw_Cpt = 0;
	DWORD dw_SelectionValueArray[100];
	DWORD dw_Choice = 0;
	DWORD dw_Dummy = 0;
	double d_RangeValueArray [100];
	DWORD dw_Type [100];
  
	/* Get the number of transducer type */
	i_Error = soap_call_MX370x__TransducerGetNbrOfType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &Response);
	if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MX370x__TransducerGetNbrOfType", 0))
		return -1;
	else
	{
		printf("\nNumber of transducer types: %d\n", Response.ulValue);

		for (dw_Cpt = 0; dw_Cpt < Response.ulValue; dw_Cpt ++)
		{
			/* Get the transducer information */
			i_Error = soap_call_MX370x__TransducerGetTypeInformation (ps_SoapEnv,pc_SOAPIPAddress, SOAP_ACTION, dw_Cpt, &TypeInfoResponse);
			if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, TypeInfoResponse.iReturnValue, 0, "soap_call_MX370x__TransducerGetTypeInformation", 0))
				return -2;				
			else
			{
				printf("\n");
				printf("Transducer Index   : %d\n", dw_Cpt);
				printf("Selection Index    : %d\n", TypeInfoResponse.ulTransducerSelectionIndex);

				dw_SelectionValueArray[dw_Cpt] = TypeInfoResponse.ulTransducerSelectionIndex;
				d_RangeValueArray[dw_Cpt] = TypeInfoResponse.dRange;
				printf("Name               : %s\n", TypeInfoResponse.pcName);
				printf("Calibration status : %d\n", TypeInfoResponse.ulCalibrationStatus);
				printf("Type value         : %d\n", TypeInfoResponse.ulType);
				dw_Type[dw_Cpt] = TypeInfoResponse.ulType;
				printf("Frequency          : %d Hz\n", TypeInfoResponse.ulFrequency);
				printf("Impedance          : %d Ohm\n", TypeInfoResponse.ulImpedance);
				printf("Nominal Voltage    : %lf V\n", TypeInfoResponse.dVeff);
				printf("Sensibility        : %lf mV/V/mm\n", TypeInfoResponse.dSensibility);
				printf("Range              : +/- %lf mm\n", TypeInfoResponse.dRange);
			}
		}

		do 
		{
			printf("\nEnter the \"Transducer Index\" you want to use (0 to %d): ", (Response.ulValue - 1));
			scanf("%d",&dw_Choice);
		}
		while (dw_Choice >= Response.ulValue);
		*pdw_TransducerTypeSelection = dw_SelectionValueArray[dw_Choice];
		*pd_Range = d_RangeValueArray[dw_Choice];
		*pdw_LVDTOrHB = dw_Type[dw_Choice];
	}
	return 0;
}


void	v_TransducerCalibration (struct soap *ps_SoapEnv,
                                 PCHAR     pc_SOAPIPAddress, 
								 WORD       w_Channel, 
								 DWORD     dw_TransducerTypeSelection, 
								 double     d_Range, 
								 double     d_CalibrationPosition,
								 DWORD     dw_Version,
								 DWORD		dw_LVDTOrHB)
{
	INT                                                i_Error         = 0;
	ULONG                                             ul_LastStatus    = 0;
	double                                             d_ChannelValue  = 0;
	struct MX370x__Response                            s_SingleResponse;
	struct MX370x__ByteArrayResponse                   s_SaveResponse;
	struct MX370x__CalibrationGetCurrentStatusResponse s_Response;


	if (dw_Version > 948)
	{
		/*************************************************************************/
		/* Start the calibration with primary open circuit diagnostic connection */
		/*************************************************************************/

		i_Error = soap_call_MX370x__CalibrationStartWithPrimaryConnection (ps_SoapEnv,
			                                                               pc_SOAPIPAddress, 
				                                                           SOAP_ACTION,
					                                                       (unsigned long) dw_TransducerTypeSelection, // Transducer type selection
						                                                   (unsigned long) w_Channel,                  // Channel selection
							                                               d_CalibrationPosition,                      // Calibration position
								                                           0,
									                                       &s_SingleResponse);

		/***********************/
		/* Test if error occur */
		/***********************/
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_SingleResponse.iReturnValue, s_SingleResponse.syserrno, "soap_call_MX370x__CalibrationStartWithPrimaryConnection", 1))
		{
			return;
		}
	}
	else
	{
		/*************************************************************************/
		/* Start the calibration with primary open circuit diagnostic connection */
		/*************************************************************************/

		i_Error = soap_call_MX370x__CalibrationStart (ps_SoapEnv,
			                                          pc_SOAPIPAddress, 
				                                      SOAP_ACTION,
					                                  (unsigned long) dw_TransducerTypeSelection, // Transducer type selection
						                              (unsigned long) w_Channel,                  // Channel selection
							                          d_CalibrationPosition,                      // Calibration position
									                  &s_SingleResponse);

		/***********************/
		/* Test if error occur */
		/***********************/
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_SingleResponse.iReturnValue, s_SingleResponse.syserrno, "soap_call_MX370x__CalibrationStart", 1))
		{
			return;
		}
	}



	while (ul_LastStatus != 6)
	{
		Sleep (10);
           
		/********************************
		 * Get calibration current status
		 *********************************/
		i_Error = soap_call_MX370x__CalibrationGetCurrentStatus (ps_SoapEnv,
			                                                     pc_SOAPIPAddress,
				                                                 SOAP_ACTION,
					                                             NULL,
						                                         &s_Response);
           
		/***********************/
		/* Test if error occur */
		/***********************/
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_Response.iReturnValue, 0, "soap_call_MX370x__CalibrationGetCurrentStatus", 1))
		{
		return;
		}


		/************************************
		 * Test if start primary calibration 
		 *************************************/
		if (((s_Response.ulStatus == 1) || ((s_Response.ulStatus == 2))) && (ul_LastStatus == 0))
		{
			printf ("\nPrimary calibration started\n");

			if (s_Response.ulStatus == 2)
			{
				ul_LastStatus = 1;
			}
		}

		/**************************************
		 * Test if primary calibration started 
		 ***************************************/
		if ((s_Response.ulStatus == 1) && (ul_LastStatus == 1))
		{
			/**************
			 * Calculate V 
			 ***************/

			d_ChannelValue = (((2.8 * 2.0) / (double) 0xFFFFFF) * (double) s_Response.ulDigitalValue) - 2.8;
              
			printf ("\rPrimary calibration %7.3fV", d_ChannelValue);
		}

		/**********************************
		 * Test if end primary calibration 
		 ***********************************/
		if ((s_Response.ulStatus == 2) && (ul_LastStatus != 2))
		{
			printf ("\nPrimary calibration conclude");
			if (dw_LVDTOrHB == _TRANSDUCER_HB)
			{
				printf ("\n\nSet the transducer connected to channel %d to mechanical position 0 mm and press any key.\n", w_Channel);
			}
			if (dw_LVDTOrHB == _TRANSDUCER_LVDT)
			{
				printf ("\n\nSet the transducer connected to channel %d so that 0 mm is displayed and press any key\n", w_Channel);
			}
		}

		/******************************************
		 * Test if wait user set transducer to 0mm 
		 *******************************************/
		if ((s_Response.ulStatus == 2) && (ul_LastStatus == 2))
		{
			/*********************************
			 * Calculate the position into mm 
			 **********************************/
			d_ChannelValue = (((d_Range * 2.0) / (double) 0xFFFFFF) * (double) s_Response.ulDigitalValue) - d_Range;
              
			printf ("\rTransducer position = %8.4fmm", d_ChannelValue);
		}

		/******************************************
		 * Test if start null position calibration 
		 *******************************************/
		if ((s_Response.ulStatus == 3) && (ul_LastStatus != 3))
		{
			printf ("\n\nNull position calibration started\n");
		}

		/************************************
		 * Test if null position calibration 
		 *************************************/
		if ((s_Response.ulStatus == 3) && (ul_LastStatus == 3))
		{
			/*********************************
			 * Calculate the position into mm 
			 **********************************/
			d_ChannelValue = (((d_Range * 2.0) / (double) 0xFFFFFF) * (double) s_Response.ulDigitalValue) - d_Range;
              
			printf ("\rTransducer position = %8.4fmm", d_ChannelValue);
		}

		/****************************************
		 * Test if end user position calibration 
		 *****************************************/
		if ((s_Response.ulStatus == 4) && (ul_LastStatus != 4))
		{
			printf ("\nNull position calibration conclude");
			printf ("\n\nSet the transducer connected to channel %d to mechanical position +%7.4f mm and press any key\n", w_Channel, d_CalibrationPosition);
		}

		/****************************************************
		 * Test if wait user set transducer to user position 
		 *****************************************************/
		if ((s_Response.ulStatus == 4) && (ul_LastStatus == 4))
		{
			/*********************************
			 * Calculate the position into mm 
			 **********************************/
			d_ChannelValue = (((d_Range * 2.0) / (double) 0xFFFFFF) * (double) s_Response.ulDigitalValue) - d_Range;
              
			printf ("\rTransducer position = %8.4fmm", d_ChannelValue);
		}

		/******************************************
		 * Test if start user position calibration 
		 *******************************************/
		if ((s_Response.ulStatus == 5) && (ul_LastStatus != 5))
		{
			printf ("\n\nUser position calibration started\n");
		}

		/************************************
		 * Test if user position calibration 
		 *************************************/
		if ((s_Response.ulStatus == 5) && (ul_LastStatus == 5))
		{
			/*********************************
			 * Calculate the position into mm 
			 **********************************/
			d_ChannelValue = (((d_Range * 2.0) / (double) 0xFFFFFF) * (double) s_Response.ulDigitalValue) - d_Range;
              
			printf ("\rTransducer position = %8.4fmm", d_ChannelValue);
		}

		/****************************************
		 * Test if end user position calibration 
		 ****************************************/
		if ((s_Response.ulStatus == 7) && (ul_LastStatus != 7))
		{
			printf ("\nUser position calibration conclude");
			printf ("\n\nConnect only one transducer and press any key\n", w_Channel);
		}


		/******************************************************
		 * Test if wait that user connect only one transducer 
		 ******************************************************/
		if ((s_Response.ulStatus == 8) && (ul_LastStatus != 8))
		{
			printf ("\n\nPrimary open line calibration started\n");
		}


		/*********************************************
		 * Test if primary open line test calibration
		 *********************************************/
		if ((s_Response.ulStatus == 8) && (ul_LastStatus == 8))
		{
			/**************
			 * Calculate V 
			 ***************/

			d_ChannelValue = (((2.8 * 2.0) / (double) 0xFFFFFF) * (double) s_Response.ulDigitalValue) - 2.8;
              
			printf ("\rPrimary open line calibration %7.3fV", d_ChannelValue);
		}

		/**************************
		 * Test if end calibration 
		 ***************************/
		if ((s_Response.ulStatus == 6) && (ul_LastStatus != 6))
		{
			/*******************
			 * Error management 
			 ********************/
			switch (s_Response.ulDigitalValue)
			{
				case 0:
						printf ("\n\nCalibration conclude");

						/**********************************
						 * Save transducer board data-base 
						 ***********************************/

						/***********************
						 * Save all transducers 
						 ************************/

						i_Error = soap_call_MX370x__DataBaseSaveTransducers(ps_SoapEnv,
									                                        pc_SOAPIPAddress, 
										                                    SOAP_ACTION, 
											                                NULL,
                                                                            &s_SaveResponse);
						if (!i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_SaveResponse.sResponse.iReturnValue, s_SaveResponse.sResponse.syserrno, "soap_call_MX370x__DataBaseSaveTransducers", 1))
						{
							printf (" Transducer data base saved");
						}
						break;

				case 1:
						/***********************
						 * Stop the calibration 
						 ************************/
						printf ("\n\n!!!User have break the calibration");
						break;

				case 2:
				case 3:
				case 4:
				case 5:
				case 6:
				case 7:
						printf ("\nCan not calibrate the primary feedback");
						break;

				case 8:
				case 9:
						printf ("\nThe 0mm position can not by found.\nRestart the calibration and select a new position for the 0mm."); 
						break;

				case 10:
				case 11:
				case 12:
				case 13:
						printf ("\nThe +%7.4fmm position can not by found.\nRestart the calibration and select a new position for the +%7.4fmm.", 
								d_CalibrationPosition,
								d_CalibrationPosition);
						break;

				case 14:
						printf ("\nPrimary short circuit occur\n");
						break;

				default :
						/****************************
						 * Format and show the error 
						 *****************************/
						printf  ("\nCalibration error %ld", s_Response.ulDigitalValue);
						break;
			}
		}

		/**************************
		 * Save the current status 
		 ***************************/
		ul_LastStatus = s_Response.ulStatus;

		/*********************/
		/* Test if key press */
		/*********************/
	   
		if (kbhit ())
		{
			/******************/
			/* Test if no ESC */
			/******************/

			if (getch () != 27)
			{
				/**********************************
				 * Test if calibration in progress 
				 ***********************************/

				if (ul_LastStatus != 6)
				{
					/*****************************
					 * Test if next step selected 
					 ******************************/

					if ((ul_LastStatus == 2) || (ul_LastStatus == 4) || (ul_LastStatus == 7))
					{
						/*****************************************
						 * Send the next calibration step command 
						 ******************************************/
                 
						soap_call_MX370x__CalibrationNextStep (ps_SoapEnv,
															   pc_SOAPIPAddress, 
															   SOAP_ACTION,
                                                               NULL,
                                                               &s_SingleResponse);
						i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_SingleResponse.iReturnValue, s_SingleResponse.syserrno, "soap_call_MX370x__CalibrationNextStep", 1);
                 
					}
				}
			}
			else
			{
				/**********************************
				 * Test if calibration in progress 
				 ***********************************/

				if (ul_LastStatus != 6)
				{
					/***********************
					 * Stop the calibration 
					 ************************/
					soap_call_MX370x__CalibrationBreak (ps_SoapEnv,
														pc_SOAPIPAddress, 
														SOAP_ACTION,
                                                        NULL,
                                                        &s_SingleResponse);
					i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_SingleResponse.iReturnValue, s_SingleResponse.syserrno, "soap_call_MX370x__CalibrationBreak", 1);
                 
				}
			}
		}
		
		/* 
		 * gSOAP doesn't release automatically the memory that 
		 * it allocates to deserialize data, that's why both 
		 * following functions have to be used to release the 
		 * allocated memory.
		 */
		soap_destroy(ps_SoapEnv);
		soap_end(ps_SoapEnv);		
	}
	return;
}


void	main(void)
	{
	CHAR                                pc_IPAddressTemp[50];
	CHAR                                pc_IPAddress    [50];
	CHAR                                pc_SOAPIPAddress[50];
	BYTE                                 b_Exit                      = 0;
	INT                                  i_Error                     = 0;
	INT                                  i_ReturnValue               = 0;
	DWORD                               dw_Dummy                     = 0;
	DWORD                               dw_Version                   = 0;
	DWORD                               dw_NbrOfChannels             = 0;
	DWORD                               dw_Cpt                       = 0;
	DWORD								dw_LVDTOrHB					 = 0;
	xsd__double                          d_Range                     = 0;
	xsd__unsignedLong                   dw_TransducerTypeSelection   = 0;
	struct soap                        *ps_SoapEnv                   = NULL;
	struct MXCommon__ByteArrayResponse   s_ByteArrayResponse;


	/**************************/
	/* Get the console handle */
	/**************************/
	ShowWindow(GetForegroundWindow (), SW_SHOWMAXIMIZED);

	/****************************/
	/* Print sample information */
	/****************************/
	printf("+---------------------------------------------------+\n");
	printf("| MX370x SAMPLE01                                   |\n");
	printf("| MX370x Calibration Program                        |\n");
	printf("+---------------------------------------------------+\n");
	printf("| <ESC> exit sample                                 |\n");
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv,pc_SOAPIPAddress , SOAP_ACTION, &dw_Dummy,&s_ByteArrayResponse);
		if (!i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_ByteArrayResponse.sResponse.iReturnValue, s_ByteArrayResponse.sResponse.syserrno, "soap_call_MXCommon__GetModuleType", 0))	   
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);
			
			if (i_UTILS_GetModuleInformations(s_ByteArrayResponse.sArray.__ptr, &dw_NbrOfChannels, &dw_Version) == 0)
			{
				printf("\t=> This is a %d transducers module\n",dw_NbrOfChannels);
				printf("\t=> Its revision is : %d \n",dw_Version);

				/* Get Transducer type selection */
				if(i_DisplayAndSelectTransducerType(ps_SoapEnv, pc_SOAPIPAddress, (DWORD *)&dw_TransducerTypeSelection, &d_Range, &dw_LVDTOrHB) == 0)
				{
					v_TransducerCalibration (ps_SoapEnv, pc_SOAPIPAddress, 0, dw_TransducerTypeSelection, d_Range, d_Range / 2, dw_Version, dw_LVDTOrHB);
				}
			}
			else
			{
				printf("find the number of channel error\n");
			}
		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

   
	/**********************************/
	/*Print End programme informations*/
	/**********************************/
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));
	}
