
/** @file Diagnostic.c
* Diagnostic : MX370x : Module Diagnostic Test
* 
*
* @author C. Guinot
* @date 04.10.2006
* 
* @version  $LastChangedRevision$
* @version $LastChangedDate$
* 
* 
* @par LICENCE
* @verbatim
*  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
*
*        ADDI-DATA GmbH 
*        Airport Boulevard B210
*        D-77836 Rheinmnster Germany 
*        Tel: +49(0)7229/1847-0 
*        Fax: +49(0)7229/1847-200
*        http://www.addi-data-com 
*        info@addi-data.com 
*
*
* @endverbatim   
*
* @par UPDATES                                                                                                          
* @verbatim                                                                                                             
*                                                                                                                       
* @endverbatim     
*
*
*/ 



/* Color management */
 #include <Windows.h>

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>

#include "..\..\..\..\Interface_Library\MX370x.nsmap"
#include "..\UTIL\utility.h"
#include "..\UTIL\socket.h"

#define S_IODRIVER 2


/* Functions help for the menue */

#define SHORT_CIRCUIT_EVENT_INFO			"\n\n\nEVENT: Short circuit on primary side detected!\n\
Please, check connections and, rearm the primary side \"6. Rearm Primary\".\n"

#define INIT_PRIMARY_CONNECTION_INFO		"\n Before to run primary connection test\n \
the module has to know what are the current\n \
channels states (transducer connected or not)\n \
The soap_call_MX370x__TransducerInitPrimaryConnectionTest function\n \
get the channels states. If a transducer is connected\n \
or removed, this function has to be called to actualize\n \
channels status.\n"

#define PRIMARY_SHORT_CIRCUIT_TEST_INFO		"\n Test if a short circuit is present on the primary side.\n \
If a short circuit is detected, check that:\n \
* OSC+ is not connected to the ground\n \
* OSC- is not connected to the ground\n \
* OSC+ is not connected to OSC-\n \
then select \"Rearm primary\".\n"

#define PRIMARY_CONNECTION_TEST_INFO		"\n Compare the channels connections states generated by\n \
soap_call_MX370x__TransducerInitPrimaryConnectionTest\n \
with the current channels connections states\n \
to determine if a connection on primary side\n \
is broken.\n"

#define REARM_PRIMARY_SIDE_INFO				"\n When a short circuit occured on the primary side,\n \
and that you solved the cause of it, the primary side has\n \
to be rearmed in order to use the module.\n \
Reason for that is because when a short circuit\n \
occured channels are disconnected to avoid damages.\n"


#define SECONDARY_CONNECTION_TEST_INFO		"\n Test if the specified channel line on the secondary\n \
side is broken.\n"

#define SECONDARY_SHORT_CIRCUIT_TEST_INFO	"\n Test if the selected channel has a short circuit on\n \
the secondary side. A short cirucit means that the channel\n \
is connected to the ground. No rearm is necessary after\n \
a secondary side short cirucit.\n"

/*******************************************************************************************************/
/*                                       UTILS Functions                                               */               
/*******************************************************************************************************/

/** Print a string with another color as the default consol color.
 *
 * @param[in] str	String to print.
 * @param[in] color Colors to apply:
 *
 *  Attribute					Meaning 
 * FOREGROUND_BLUE				Text color contains blue. 
 * FOREGROUND_GREEN				Text color contains green. 
 * FOREGROUND_RED				Text color contains red. 
 * FOREGROUND_INTENSITY			Text color is intensified. 
 * BACKGROUND_BLUE				Background color contains blue. 
 * BACKGROUND_GREEN				Background color contains green. 
 * BACKGROUND_RED				Background color contains red. 
 * BACKGROUND_INTENSITY			Background color is intensified. 
 * COMMON_LVB_LEADING_BYTE		Leading byte. 
 * COMMON_LVB_TRAILING_BYTE		Trailing byte. 
 * COMMON_LVB_GRID_HORIZONTAL	Top horizontal. 
 * COMMON_LVB_GRID_LVERTICAL	Left vertical. 
 * COMMON_LVB_GRID_RVERTICAL	Right vertical. 
 * COMMON_LVB_REVERSE_VIDEO		Reverse foreground and background attributes. 
 * COMMON_LVB_UNDERSCORE		Underscore. 
 */
void ColorPrint (char *str, WORD color)
{
	CONSOLE_SCREEN_BUFFER_INFO	csbiInfo;
	WORD						wOldColorAttrs;
	BYTE						b_UseColor;
	HANDLE						hStdout;

	hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
	if (hStdout != INVALID_HANDLE_VALUE)
	{
		if (GetConsoleScreenBufferInfo(hStdout, &csbiInfo) != 0) 
		{
			wOldColorAttrs = csbiInfo.wAttributes;
			b_UseColor = 1; /* OK, color will be used */
			/* Change the color of the text when an event occured */
			SetConsoleTextAttribute (hStdout, color);
		}
	}
	
	printf("%s", str);

	/* Set back the default consol colors */							
	if (b_UseColor)
		SetConsoleTextAttribute (hStdout, wOldColorAttrs);
}

/** Get the current consol colors and set the new colors.
 *
 * @param[in] color Colors to apply:
 *
 *  Attribute					Meaning 
 * FOREGROUND_BLUE				Text color contains blue. 
 * FOREGROUND_GREEN				Text color contains green. 
 * FOREGROUND_RED				Text color contains red. 
 * FOREGROUND_INTENSITY			Text color is intensified. 
 * BACKGROUND_BLUE				Background color contains blue. 
 * BACKGROUND_GREEN				Background color contains green. 
 * BACKGROUND_RED				Background color contains red. 
 * BACKGROUND_INTENSITY			Background color is intensified. 
 * COMMON_LVB_LEADING_BYTE		Leading byte. 
 * COMMON_LVB_TRAILING_BYTE		Trailing byte. 
 * COMMON_LVB_GRID_HORIZONTAL	Top horizontal. 
 * COMMON_LVB_GRID_LVERTICAL	Left vertical. 
 * COMMON_LVB_GRID_RVERTICAL	Right vertical. 
 * COMMON_LVB_REVERSE_VIDEO		Reverse foreground and background attributes. 
 * COMMON_LVB_UNDERSCORE		Underscore. 
 * 
 * @retval >0 The previous consol colors.
 * @retval <0 No color management available.
 */
long SetColor (long color)
{
	CONSOLE_SCREEN_BUFFER_INFO	csbiInfo;
	HANDLE						hStdout;
	WORD						c;

	hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
	if (hStdout == INVALID_HANDLE_VALUE)
		return -1;

	if (GetConsoleScreenBufferInfo(hStdout, &csbiInfo) == 0) 
		return -1;

	c = (WORD)color;

	/* Change the color of the text when an event occured */
	SetConsoleTextAttribute (hStdout, c);

	return csbiInfo.wAttributes;
}

/** This function has to be used in combination of
 *  SetColor to set the prvious consol colrs.
 *
 * @param[in] This has to be the return value of SetColor.
 *
 * @retval 0 No error.
 * @retval <0 No color management available.
 */
long ResetColor (long color)
{
	HANDLE hStdout;
	WORD c;

	if (color > 0)
	{
		hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
		if (hStdout == INVALID_HANDLE_VALUE)
			return -1;

		c = (WORD)color;

		/* Change the color of the text when an event occured */
		SetConsoleTextAttribute (hStdout, c);
	}

	return 0;
}

INT i_UTILS_GetTheNumberOfChannels (PCHAR pc_ModuleType, DWORD * pdw_NbrOfChannels)
{
	DWORD dw_Cpt = 0;
	CHAR pc_NbrOfChannels [5];
	DWORD dw_NbrOfChannelIndex = 0;

	/* check presence of the substring "UNKNOWN" indicating a not yet configured module */
	if (strstr(pc_ModuleType,"UNKNOWN"))
	{
		fprintf(stderr,"! Remote module not configured\n");
		return -1;
	}

	/* If DIO system, the pattern is not the same */
	if (strstr(pc_ModuleType, "DIO"))
	{
		/* module is like: MSX-E3701-DIO-HB-XX or MSX-E3701-DIO-LVDT-XX*/	
		/* get a pointer to HB or LVDT */
		PCHAR beg = strstr(pc_ModuleType, "LVDT");
		if ( ! beg )
			beg = strstr(pc_ModuleType, "HB");
		if ( ! beg )
		{
			fprintf(stderr,"! Unknown system: %s \n", pc_ModuleType);
			return -1;
		}

		/* loop until we find the first "-" caracter */
		for (dw_Cpt = 0; dw_Cpt < strlen(beg);dw_Cpt++)
		{
			if (beg [dw_Cpt] == '-') 
			{
				dw_Cpt++;
				break;
			}
		}

		/* now we have the number of channels */
		for ( ; dw_Cpt < strlen(beg); dw_Cpt++)
		{
			if ( (beg [dw_Cpt] < '0') ||(beg [dw_Cpt] > '9') )
			{
				break;
			}
			pc_NbrOfChannels[dw_NbrOfChannelIndex] = beg[dw_Cpt];
			dw_NbrOfChannelIndex++;
		}

		/* check */
		if ( dw_NbrOfChannelIndex == 0 )
			return -1;

		/* convert and return */
		pc_NbrOfChannels[dw_NbrOfChannelIndex] = '\0';
		*pdw_NbrOfChannels = atol (pc_NbrOfChannels);
		return 0;
	}

	/* Normal systems */
	for (dw_Cpt = 0; dw_Cpt < strlen(pc_ModuleType);dw_Cpt++)
	{
		if (pc_ModuleType [dw_Cpt] == '-') 
		{
			dw_Cpt++;
			break;
		}
	}

	if (dw_Cpt < strlen(pc_ModuleType))
	{
		for (dw_NbrOfChannelIndex = 0; dw_NbrOfChannelIndex < (strlen(pc_ModuleType) - dw_Cpt ); dw_NbrOfChannelIndex ++)
		{
			if (pc_ModuleType [dw_Cpt + dw_NbrOfChannelIndex] == '-')
				break;
			else
				pc_NbrOfChannels [dw_NbrOfChannelIndex] = pc_ModuleType [dw_Cpt + dw_NbrOfChannelIndex];

		}
		pc_NbrOfChannels[dw_NbrOfChannelIndex] = '\0';

		if (dw_NbrOfChannelIndex < (strlen(pc_ModuleType) - dw_Cpt ))
			*pdw_NbrOfChannels = atol (pc_NbrOfChannels);
		else
			return -1;
	}
	else
		return -1;
	
	return 0;
}


typedef struct
      {
      BYTE							b_Exit;
      SOCKET						s_ClientSocket;
      }str_ThreadInfo;


DWORD WINAPI dw_SocketThreadProc (LPVOID lpParameter)
{
	/***********************
	 * Varibale declaration 
	 ************************/

	str_ThreadInfo * ps_ThreadInfo;
	unsigned char   b_EventFormat        = 0;
	unsigned char   b_NbrBytesToRead     = 0;
	unsigned char *pb_Buffer             = NULL;
	int             i_Error              = 0;
	int             i_ReturnValue        = 0;
	unsigned long  dw_NbrOfReceiveValues = 0;
	unsigned long  dw_RequestValue       = 0;
        
	/**************************
	 * Variable initialisation 
	 ***************************/
	ps_ThreadInfo = (str_ThreadInfo *) lpParameter;
        
	do
	{
		/*Get the event format*/
		i_ReturnValue = recv (ps_ThreadInfo->s_ClientSocket, &b_EventFormat, 1, 0);
                    
		if (i_ReturnValue != SOCKET_ERROR)
		{
			if (b_EventFormat == 0)
			{
				/*Get the nbr of byte to read*/
				i_ReturnValue = recv (ps_ThreadInfo->s_ClientSocket, &b_NbrBytesToRead, 1, 0);
                          
				if (i_ReturnValue != SOCKET_ERROR)
				{
					dw_RequestValue       = b_NbrBytesToRead;
					dw_NbrOfReceiveValues = 0;
                             
					pb_Buffer = malloc (b_NbrBytesToRead);
                             
					do
					{
						/*Get the event*/
						i_ReturnValue = recv (ps_ThreadInfo->s_ClientSocket, &pb_Buffer[dw_NbrOfReceiveValues], dw_RequestValue, 0);
                
						if ((i_ReturnValue != SOCKET_ERROR))
						{
							dw_NbrOfReceiveValues = dw_NbrOfReceiveValues + i_ReturnValue;
							dw_RequestValue = dw_RequestValue - i_ReturnValue;
						}
					} while ((dw_RequestValue > 0) && (i_ReturnValue != SOCKET_ERROR));

					if ((i_ReturnValue != SOCKET_ERROR) && (pb_Buffer [0] == S_IODRIVER))
					{						
						/* Test if short circuit event on primary side */ 
						if (pb_Buffer[1] == 2)
						{				
							ColorPrint (SHORT_CIRCUIT_EVENT_INFO, FOREGROUND_RED | FOREGROUND_INTENSITY);
						}
					}
				}
				else
				{
				ps_ThreadInfo->b_Exit = 1;
				}
			}
		}
		else
		{
			ps_ThreadInfo->b_Exit = 1;
		}
	} while (ps_ThreadInfo->b_Exit == 0);

	/*********************
	 * Return from thread 
	 **********************/
	return (0);
}


void main(void)
{
	char                                       c_IPAddressTemp[50];
	char                                       c_IPAddress    [50];
	char									   c_SOAPIPAddress[50];
	WORD                                       w_Choice        = 0;
	int                                        i_Error         = 0;
	int                                        i_ReturnValue   = 0;
	unsigned long                             dw_Dummy         = 0;
	unsigned long                             dw_Channel       = 0;
	unsigned long                             dw_NbrOfChannels = 0;
	DWORD                                     dw_ThreadId      = 0;
	SOCKET                                     s_ClientSocket;
	HANDLE                                     h_TreadHandle   = 0;
	struct MXCommon__ByteArrayResponse         s_ByteArrayResponse;
	struct MX370x__Response                    s_Response;
	struct MX370x__unsignedlongDefaultResponse ul_Response;
	struct soap                               *ps_SoapEnv = NULL;
    str_ThreadInfo                             s_ThreadInfo;
	char 										c_PrimaryShortCircuitStatus      [2][30] = {"Short circuit", "No short circuit"};
	char 										c_RearmePrimaryShortCircuitStatus[2][30] = {"Short circuit persist", "Rearme OK"};
	char 										c_PrimaryOpenCircuitStatus       [2][30] = {"Any connection open", "All connections OK"};
	char										c_SecondaryOpenCircuitStatus     [2][30] = {"Connection open", "Connection OK"};
	char										c_SecondaryShortCircuitStatus    [2][30] = {"Short circuit", "No short circuit"};
	long										l_oldColor = 0;
	


	ShowWindow(GetForegroundWindow (), SW_SHOWMAXIMIZED);   

       
	/****************************/
	/* Print sample information */
	/****************************/

	printf("+---------------------------------------------------+\n");
	printf("| MSX-E370x Diagnostic                              |\n");
	printf("| ====================                              |\n");
	printf("|                                                   |\n");
	printf("| This sample demonstrates how to use diagnostic    |\n");
	printf("| functions. Please follow instructions.            |\n");
	printf("|                                                   |\n");
	printf("| ");
	ColorPrint("- Text in light red are event from the module.    ", FOREGROUND_RED | FOREGROUND_INTENSITY);
	printf("|\n");
	printf("| ");
	ColorPrint("- Text in green are helps about functions         ", FOREGROUND_GREEN);
	printf("|\n");
	printf("| ");
	ColorPrint("- Text in dark red are functions execution results", FOREGROUND_RED);
	printf("|\n");
	printf("+---------------------------------------------------+\n\n");

        
	/* Initialise the SOAP connection */
        
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(c_IPAddressTemp);
           
		v_UTILS_ClearNullInIPAddress(c_IPAddressTemp,c_IPAddress);
		sprintf(c_SOAPIPAddress,"%s:5555", c_IPAddress);
		printf("SOAP IP Address: %s\n", c_SOAPIPAddress);
           
		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv,c_SOAPIPAddress , SOAP_ACTION, &dw_Dummy, &s_ByteArrayResponse);
           
		if (!i_CheckAndPrintError (ps_SoapEnv, c_SOAPIPAddress, SOAP_ACTION, i_Error, s_ByteArrayResponse.sResponse.iReturnValue, s_ByteArrayResponse.sResponse.syserrno, "soap_call_MXCommon__GetModuleType", 0))
		{
            ps_SoapEnv->send_timeout    = 30;
            ps_SoapEnv->recv_timeout    = 30;
            ps_SoapEnv->accept_timeout  = 30;
            ps_SoapEnv->connect_timeout = 30;

			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);

			/* Get the number of channels */

			i_UTILS_GetTheNumberOfChannels(s_ByteArrayResponse.sArray.__ptr,&dw_NbrOfChannels);
              
              
			/* Initialise the socket to receive the event */
              
			if (i_Socket_ClientOpen(c_IPAddress,6363,&s_ClientSocket) == 0)
			{

				s_ThreadInfo.s_ClientSocket = s_ClientSocket;
				s_ThreadInfo.b_Exit = 0;
                 
				/**************************
				 * Creat the socket thread
				 **************************/
				h_TreadHandle = CreateThread (NULL,
                                              0,
                                              dw_SocketThreadProc,
                                              &s_ThreadInfo,
                                              0,
                                              &dw_ThreadId);
                 
				Sleep (100);				

				/* Test if a short circuit is present on the primary side.
				 * If a short circuit is detected, check that:
				 * OSC+ is not connected to the ground
				 * OSC- is not connected to the ground
				 * OSC+ is not connected to OSC-
				 * then select "Rearm primary".
				 */
				ColorPrint(PRIMARY_SHORT_CIRCUIT_TEST_INFO, FOREGROUND_GREEN);
				i_Error = soap_call_MX370x__TransducerTestPrimaryShortCircuit(ps_SoapEnv,c_SOAPIPAddress, SOAP_ACTION,&dw_Dummy, &ul_Response);
				if (!i_CheckAndPrintError (ps_SoapEnv, c_SOAPIPAddress, SOAP_ACTION, i_Error, ul_Response.sResponse.iReturnValue, ul_Response.sResponse.syserrno, "soap_call_MX370x__TransducerInitPrimaryConnectionTest", 0))
				{
					l_oldColor = SetColor (FOREGROUND_RED);
					printf("\n TransducerTestPrimaryShortCircuit OK Result: %lu -> %s",ul_Response.ulValue, c_PrimaryShortCircuitStatus[ul_Response.ulValue]);
					ResetColor (l_oldColor);
				}

				/* If no short circuit on the primary side */	
				if (ul_Response.ulValue)
				{
					/* First things to be done (before to start primary connection test) is to get the current channels on which transducers are connected.
					 * This is done by calling soap_call_MX370x__TransducerInitPrimaryConnectionTest that maps the current
					 * connected channels. This "map" will be used by soap_call_MX370x__TransducerTestPrimaryConnection to determine 
					 * if a connection has changed. If you plane to use soap_call_MX370x__TransducerTestPrimaryConnection, 
					 * this function has to be called each time that a transducer is connected or removed 
					 * from the module.
					 */
					ColorPrint(INIT_PRIMARY_CONNECTION_INFO, FOREGROUND_GREEN);
					l_oldColor = SetColor (FOREGROUND_RED);
					
					i_Error = soap_call_MX370x__TransducerInitPrimaryConnectionTest(ps_SoapEnv,c_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_Response);
					i_CheckAndPrintError (ps_SoapEnv, c_SOAPIPAddress, SOAP_ACTION, i_Error, s_Response.iReturnValue, s_Response.syserrno, "soap_call_MX370x__TransducerInitPrimaryConnectionTest", 0);
					ResetColor (l_oldColor);
				}
                 
				do
				{

					/* 
					 * If you are writing a program that makes tests automatically,
					 * you have to call functions in the following order:
					 *
					 * Primary short circuit test (Rearm primary side, if necessary)
					 * Secondary connection test
					 * Secondary short circuit test
					 * Init primary connection test
					 * Primary connection test
					 */
					printf("\n\n Select an action:");
					printf("\n-----------------------------------");
					printf("\n 1. Init primary connection test"); 
					ColorPrint(INIT_PRIMARY_CONNECTION_INFO, FOREGROUND_GREEN);

					printf("\n 2. Secondary connection test");
					ColorPrint(SECONDARY_CONNECTION_TEST_INFO, FOREGROUND_GREEN);
					
					printf("\n 3. Secondary short circuit test");				
					ColorPrint(SECONDARY_SHORT_CIRCUIT_TEST_INFO, FOREGROUND_GREEN);
					
					printf("\n 4. Primary connection test");
					ColorPrint(PRIMARY_CONNECTION_TEST_INFO, FOREGROUND_GREEN);					
					
					printf("\n 5. Primary short circuit test");
					ColorPrint(PRIMARY_SHORT_CIRCUIT_TEST_INFO, FOREGROUND_GREEN);
					
					printf("\n 6. Rearm primary side"); 
					ColorPrint(REARM_PRIMARY_SIDE_INFO, FOREGROUND_GREEN);
									
					printf("\n 7. Exit");
					printf("\n Your choice : ");

					do
					{
						if (kbhit ())
						{
							w_Choice = getch ();
						}
						else if (s_ThreadInfo.b_Exit != 0)
						{
							w_Choice = '7';
						}
						else
						{
							Sleep (100);
						}
					} while ((w_Choice < '1') || (w_Choice > '7'));

					printf ("%c\n", w_Choice);
					w_Choice = w_Choice - '0';

					switch (w_Choice)
					{
					case 1:
							/* First things to be done is to get the current channels on which transducers are connected.
							 * This is done by calling soap_call_MX370x__TransducerInitPrimaryConnectionTest that maps the current
							 * connected channels. This "map" will be used by soap_call_MX370x__TransducerTestPrimaryConnection to determine 
							 * if a connection has changed. If you plane to use soap_call_MX370x__TransducerTestPrimaryConnection, 
							 * this function has to be called each time that a transducer is connected or removed 
							 * from the module.
							 */
							l_oldColor = SetColor (FOREGROUND_RED);
							i_Error = soap_call_MX370x__TransducerInitPrimaryConnectionTest(ps_SoapEnv,c_SOAPIPAddress, SOAP_ACTION,&dw_Dummy, &s_Response);
							i_CheckAndPrintError (ps_SoapEnv, c_SOAPIPAddress, SOAP_ACTION, i_Error, s_Response.iReturnValue, s_Response.syserrno, "soap_call_MX370x__TransducerInitPrimaryConnectionTest", 0);
							ResetColor (l_oldColor);
							break;

					case 4:
							/* Compare the channels connections states generated by soap_call_MX370x__TransducerInitPrimaryConnectionTest
							 * with the current channels connections states to determine if a connection on primary side 
							 * is broken.
							 */
							l_oldColor = SetColor (FOREGROUND_RED);
							i_Error = soap_call_MX370x__TransducerTestPrimaryConnection(ps_SoapEnv,c_SOAPIPAddress, SOAP_ACTION,&dw_Dummy, &ul_Response);
							if (!i_CheckAndPrintError (ps_SoapEnv, c_SOAPIPAddress, SOAP_ACTION, i_Error, ul_Response.sResponse.iReturnValue, ul_Response.sResponse.syserrno, "soap_call_MX370x__TransducerTestPrimaryConnection", 0))
							{
								printf(" Result: %lu -> %s",ul_Response.ulValue, c_PrimaryOpenCircuitStatus[ul_Response.ulValue]);
							}
							ResetColor (l_oldColor);
							break;

					case 5:
							/* Test if a short circuit is present on the primary side.
							 * If a short circuit is detected, check that:
							 * OSC+ is not connected to the ground
							 * OSC- is not connected to the ground
							 * OSC+ is not connected to OSC-
							 * then select "Rearm primary".
							 */
							l_oldColor = SetColor (FOREGROUND_RED);
							i_Error = soap_call_MX370x__TransducerTestPrimaryShortCircuit(ps_SoapEnv,c_SOAPIPAddress, SOAP_ACTION,&dw_Dummy, &ul_Response);
							if (!i_CheckAndPrintError (ps_SoapEnv, c_SOAPIPAddress, SOAP_ACTION, i_Error, ul_Response.sResponse.iReturnValue, ul_Response.sResponse.syserrno, "soap_call_MX370x__TransducerTestPrimaryShortCircuit", 0))
							{
								printf(" Result: %lu -> %s",ul_Response.ulValue, c_PrimaryShortCircuitStatus[ul_Response.ulValue]);
							}
							ResetColor (l_oldColor);
							break;

					case 6:
							/* When a short circuit occured on the primary side, and that you solved 
							 * the cause of it, the primary side has to be rearmed in order to use the 
							 * module. Reason for that is because when a short circuit occured channels
							 * are disconnected to avoid damages.
							 */
							l_oldColor = SetColor (FOREGROUND_RED);
							i_Error = soap_call_MX370x__TransducerRearmPrimary(ps_SoapEnv,c_SOAPIPAddress, SOAP_ACTION,&dw_Dummy, &ul_Response);
							if (!i_CheckAndPrintError (ps_SoapEnv, c_SOAPIPAddress, SOAP_ACTION, i_Error, ul_Response.sResponse.iReturnValue, ul_Response.sResponse.syserrno, "soap_call_MX370x__TransducerRearmPrimary", 0))
							{
								printf(" Result: %lu -> %s",ul_Response.ulValue, c_RearmePrimaryShortCircuitStatus[ul_Response.ulValue]);
							}
							ResetColor (l_oldColor);
							break;

					case 2:							
							/* Test if the specified channel line on the secondary side is broken.
							 */
							do 
							{
								printf("\n Select the channel to test (1 to %lu):",dw_NbrOfChannels);
								scanf("%lu",&dw_Channel);
							} while((dw_Channel > dw_NbrOfChannels)||(dw_Channel < 1));
							l_oldColor = SetColor (FOREGROUND_RED);
							i_Error = soap_call_MX370x__TransducerTestSecondaryConnection(ps_SoapEnv,c_SOAPIPAddress, SOAP_ACTION,dw_Channel-1, &ul_Response);
							if (!i_CheckAndPrintError (ps_SoapEnv, c_SOAPIPAddress, SOAP_ACTION, i_Error, ul_Response.sResponse.iReturnValue, ul_Response.sResponse.syserrno, "soap_call_MX370x__TransducerTestSecondaryConnection", 0))
							{
								printf(" Result: %lu -> %s",ul_Response.ulValue, c_SecondaryOpenCircuitStatus[ul_Response.ulValue]);
							}
							ResetColor (l_oldColor);
							break;

					case 3:
							/* Test if the selected channel has a short circuit on the secondary side.
							 * A short cirucit means that the channel is connected to the ground.
							 * No ream is necessary after a secondary side short cirucit.
							 */
							do 
							{
								printf("\n Select the channel to test (1 to %lu):",dw_NbrOfChannels);
								scanf("%lu",&dw_Channel);
							} while((dw_Channel > dw_NbrOfChannels)||(dw_Channel < 1));
							l_oldColor = SetColor (FOREGROUND_RED);
							i_Error = soap_call_MX370x__TransducerTestSecondaryShortCircuit(ps_SoapEnv,c_SOAPIPAddress, SOAP_ACTION,dw_Channel-1, &ul_Response);
							if (!i_CheckAndPrintError (ps_SoapEnv, c_SOAPIPAddress, SOAP_ACTION, i_Error, ul_Response.sResponse.iReturnValue, ul_Response.sResponse.syserrno, "soap_call_MX370x__TransducerTestSecondaryShortCircuit", 0))
							{
								printf(" Result: %lu -> %s",ul_Response.ulValue, c_SecondaryShortCircuitStatus[ul_Response.ulValue]);
							}
							ResetColor (l_oldColor);
							break;

					case 7:
							s_ThreadInfo.b_Exit = 1;

							/* Close the event Socket */
							i_Socket_ClientClose (s_ClientSocket);
							break;

					default :
							break;
					}
					
					/* 
					 * gSOAP doesn't release automatically the memory that 
					 * it allocates to deserialize data, that's why both 
					 * following functions have to be used to release the 
					 * allocated memory.
					 */
					soap_destroy(ps_SoapEnv);
					soap_end(ps_SoapEnv);					
					
				} while (w_Choice != 7);
                 
				/***************************
				 * Test if thread installed 
				 ****************************/
                 
				if ((h_TreadHandle != 0))
				{
					/*************************
					 * Close the tread handle 
					 **************************/
					CloseHandle (h_TreadHandle);
					Sleep (100);
				}
			}
			else
			{
				printf("\ni_Socket_ClientOpen Error\n");
			}
		}

		/* Close the SOAP connection */
		i_SOAP_Close(ps_SoapEnv);
        
		}
	/**********************************/
	/*Print End programme informations*/
	/**********************************/
        
	printf("\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");
        
	while (!(kbhit()));
	getch ();
}
