
/** @file MinMax.c
* MinMax : MX370x : Min/Max acquisition
* 
*
* @author C. Guinot
* @date 06.04.2006
* 
* @version  $LastChangedRevision$
* @version $LastChangedDate$
* 
* 
* @par LICENCE
*
* @verbatim
*  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
*        
*        ADDI-DATA GmbH 
*        Airport Boulevard B210
*        D-77836 Rheinmnster Germany 
*        Tel: +49(0)7229/1847-0 
*        Fax: +49(0)7229/1847-200
*        http://www.addi-data-com 
*        info@addi-data.com 
*        
*
* @endverbatim   
*
* @par UPDATES                                                                                                          
* @verbatim                                                                                                             
*  Date      : 01.06.06                                                                                                 
*  Author        : C. Guinot                                                                                            
*  Description   : Remove the "_" character in the SOAP parameter name
*                                                                                                                       
* @endverbatim     
*
*
*/ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>


#include "..\..\..\..\Interface_Library\MX370x.nsmap"
#include "..\UTIL\utility.h"
#include "..\UTIL\socket.h"


/*******************************************************************************************************/
/*                                       UTILS Functions                                               */               
/*******************************************************************************************************/

INT i_UTILS_GetTheNumberOfChannels (PCHAR pc_ModuleType, DWORD * pdw_NbrOfChannels)
{
	DWORD dw_Cpt = 0;
	CHAR pc_NbrOfChannels [5];
	DWORD dw_NbrOfChannelIndex = 0;

	/* check presence of the substring "UNKNOWN" indicating a not yet configured module */
	if (strstr(pc_ModuleType,"UNKNOWN"))
	{
		fprintf(stderr,"! Remote module not configured\n");
		return -1;
	}

	/* If DIO system, the pattern is not the same */
	if (strstr(pc_ModuleType, "DIO"))
	{
		/* module is like: MSX-E3701-DIO-HB-XX or MSX-E3701-DIO-LVDT-XX*/	
		/* get a pointer to HB or LVDT */
		PCHAR beg = strstr(pc_ModuleType, "LVDT");
		if ( ! beg )
			beg = strstr(pc_ModuleType, "HB");
		if ( ! beg )
		{
			fprintf(stderr,"! Unknown system: %s \n", pc_ModuleType);
			return -1;
		}

		/* loop until we find the first "-" caracter */
		for (dw_Cpt = 0; dw_Cpt < strlen(beg);dw_Cpt++)
		{
			if (beg [dw_Cpt] == '-') 
			{
				dw_Cpt++;
				break;
			}
		}

		/* now we have the number of channels */
		for ( ; dw_Cpt < strlen(beg); dw_Cpt++)
		{
			if ( (beg [dw_Cpt] < '0') ||(beg [dw_Cpt] > '9') )
			{
				break;
			}
			pc_NbrOfChannels[dw_NbrOfChannelIndex] = beg[dw_Cpt];
			dw_NbrOfChannelIndex++;
		}

		/* check */
		if ( dw_NbrOfChannelIndex == 0 )
			return -1;

		/* convert and return */
		pc_NbrOfChannels[dw_NbrOfChannelIndex] = '\0';
		*pdw_NbrOfChannels = atol (pc_NbrOfChannels);
		return 0;
	}

	/* Normal systems */
	for (dw_Cpt = 0; dw_Cpt < strlen(pc_ModuleType);dw_Cpt++)
	{
		if (pc_ModuleType [dw_Cpt] == '-') 
		{
			dw_Cpt++;
			break;
		}
	}

	if (dw_Cpt < strlen(pc_ModuleType))
	{
		for (dw_NbrOfChannelIndex = 0; dw_NbrOfChannelIndex < (strlen(pc_ModuleType) - dw_Cpt ); dw_NbrOfChannelIndex ++)
		{
			if (pc_ModuleType [dw_Cpt + dw_NbrOfChannelIndex] == '-')
				break;
			else
				pc_NbrOfChannels [dw_NbrOfChannelIndex] = pc_ModuleType [dw_Cpt + dw_NbrOfChannelIndex];

		}
		pc_NbrOfChannels[dw_NbrOfChannelIndex] = '\0';

		if (dw_NbrOfChannelIndex < (strlen(pc_ModuleType) - dw_Cpt ))
			*pdw_NbrOfChannels = atol (pc_NbrOfChannels);
		else
			return -1;
	}
	else
		return -1;
	
	return 0;
}


int i_DisplayAndSelectTransducerType(struct soap *ps_SoapEnv,PCHAR pc_SOAPIPAddress,DWORD * pdw_TransducerTypeSelection)
        {
        struct MX370x__unsignedlongResponse Response;
        struct MX370x__TransducerGetTypeInformationResponse TypeInfoResponse;
        
        INT i_Error = 0;
        DWORD dw_Cpt = 0;
        DWORD pdw_SelectionValueArray[100];
        DWORD dw_Choice = 0;
        DWORD dw_Dummy = 0;
        
        /* Get the number of transducer type */
        i_Error = soap_call_MX370x__TransducerGetNbrOfType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &Response);
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MX370x__TransducerGetNbrOfType", 0))
			return -1;
		else
           {
           printf("Number of transducer types: %d\n", Response.ulValue);
           
           for (dw_Cpt = 0; dw_Cpt < Response.ulValue; dw_Cpt ++)
              {
              /* Get the transducer information */
              i_Error = soap_call_MX370x__TransducerGetTypeInformation (ps_SoapEnv,pc_SOAPIPAddress, SOAP_ACTION, dw_Cpt, &TypeInfoResponse);
			  if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, TypeInfoResponse.iReturnValue, 0, "soap_call_MX370x__TransducerGetTypeInformation", 0))
				  return -2;
			  else
			  {
                 printf("\n");
                         printf("Transducer Index: %d\n", dw_Cpt);
                         printf("Selection Index: %d\n", TypeInfoResponse.ulTransducerSelectionIndex);
                 
                         pdw_SelectionValueArray[dw_Cpt] = TypeInfoResponse.ulTransducerSelectionIndex;
                         printf("Name: %s\n", TypeInfoResponse.pcName);
                         printf("Calibration status: %d\n", TypeInfoResponse.ulCalibrationStatus);
                         printf("Type value: %d\n", TypeInfoResponse.ulType);
                         printf("Frequency: %d Hz\n", TypeInfoResponse.ulFrequency);
                         printf("Impedance: %d Ohm\n", TypeInfoResponse.ulImpedance);
                         printf("Nominal Voltage: %lf V\n", TypeInfoResponse.dVeff);
                         printf("Sensibility: %lf mV/V/mm\n", TypeInfoResponse.dSensibility);
                         printf("Range: +/- %lf mm\n", TypeInfoResponse.dRange);
              
                 }
              }
           do
              {
              printf("\nEnter the index of the transducer type do You want ot use (0 to %d): ", (Response.ulValue - 1));
                      scanf("%d",&dw_Choice);
           
              } while (dw_Choice >= Response.ulValue);
           *pdw_TransducerTypeSelection = pdw_SelectionValueArray[dw_Choice];
        
           }
        return 0;

        }


void GetMinMaxAcquisitionParameters(unsigned long dw_NbrOfChannelsMax, 
                                    unsigned long *pdw_ChannelMask,
                                    unsigned long *pdw_DivisionFactor,
                                    unsigned long *pdw_StopChannelMask,
                                    unsigned long *pdw_StopCondition,
                                    unsigned long *pdw_StopValue)
        {
        do
           {
           printf("\nEnter the channel mask (in Hex): ");
           scanf("%X", pdw_ChannelMask);
        
           } while ((*pdw_ChannelMask == 0) || (*pdw_ChannelMask > (pow(2,dw_NbrOfChannelsMax) - 1)));
        do
           {
           printf("\nEnter the division factor (5 to 255): ");
                   scanf("%d", pdw_DivisionFactor);
        
           } while ((*pdw_DivisionFactor < 5) || (*pdw_DivisionFactor > 255));
        do
           {
           printf("\nEnter the Stop channel mask (in Hex): ");
           scanf("%X", pdw_StopChannelMask);
        
           } while (*pdw_StopChannelMask > (pow(2,dw_NbrOfChannelsMax) - 1));
        if (*pdw_StopChannelMask != 0)
           {
           do
              {
              printf("\nEnter the Stop condition (1:<, 2:>) :");
              scanf("%X", pdw_StopCondition);
           
              } while ((*pdw_StopCondition == 0) || (*pdw_StopCondition > 2));
           do
              {
              printf("\nEnter the Stop compare value (in Hex, max = FFFFFF) : ");
              scanf("%X", pdw_StopValue);
           
              } while ((*pdw_StopValue > 0xFFFFFF));
           }
        }


void main(void)
        {
        int i_Error = 0;
        struct soap *ps_SoapEnv = NULL;
        struct MXCommon__ByteArrayResponse  s_ByteArrayResponse;
        char pc_IPAddressTemp[50];
        char pc_IPAddress[50];
        char pc_SOAPIPAddress[50];
        xsd__unsignedLong dw_TransducerTypeSelection = 0;
        unsigned long dw_Dummy = 0;
        
        unsigned long dw_NbrOfChannelsMax = 0;
        unsigned long dw_DivisionFactor = 0;
        unsigned long dw_ChannelMask = 0;
        unsigned long dw_StopChannelMask = 0;
        unsigned long dw_StopCondition = 0;
        unsigned long dw_StopValue = 0;
        
        struct MX370x__Response Response;
        struct MX370x__TransducerGetMinMaxStatusResponse s_GetMinMaxStatusResponse;
        
        
        unsigned long dw_NumberOfChannelsToAcquire = 0;
        unsigned long dw_NbrOfValuesToReceive = 0;
        unsigned long dw_RequestValue = 0;
        unsigned long dw_NbrOfReceiveValues = 0;
        unsigned long dw_Cpt = 0;
        int i_ReturnValue = 0;
        unsigned char b_Exit = 0;
        
        
        
        
        /**************************/
        /* Get the console handle */
        /**************************/

        ShowWindow(GetForegroundWindow (), SW_SHOWMAXIMIZED);
        
        /****************************/
        /* Print sample information */
        /****************************/
        printf("+---------------------------------------------------+\n");
        printf("| MX370x MinMax                                     |\n");
        printf("| MX370x Min Max Test Program                       |\n");
        printf("+---------------------------------------------------+\n");
        printf("| <ESC> exit sample                                 |\n");
        printf("+---------------------------------------------------+\n");
        printf("\n");
        
        /* Initialise the SOAP connection */
        
        if ((i_SOAP_Init(&ps_SoapEnv) == 0))
           {
           printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
           gets(pc_IPAddressTemp);
           
           v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
           sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
           printf("SOAP IP Address: %s\n", pc_SOAPIPAddress);
           
           i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv,pc_SOAPIPAddress , SOAP_ACTION, &dw_Dummy, &s_ByteArrayResponse);           
		   if (!i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_ByteArrayResponse.sResponse.iReturnValue, s_ByteArrayResponse.sResponse.syserrno, "soap_call_MXCommon__GetModuleType", 0))
              {
              printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);
              
              if (i_UTILS_GetTheNumberOfChannels(s_ByteArrayResponse.sArray.__ptr, &dw_NbrOfChannelsMax) == 0)
                 {
					printf("Number of transucer channels: %d \n", dw_NbrOfChannelsMax);

                 /* Get Transducer type selection */
                 
                 if (i_DisplayAndSelectTransducerType(ps_SoapEnv, pc_SOAPIPAddress, (DWORD *)&dw_TransducerTypeSelection) == 0)
                    {
                    /* Get the Min/Max initialisation parameter */
                    GetMinMaxAcquisitionParameters( dw_NbrOfChannelsMax,
                                                    &dw_ChannelMask,
                                                    &dw_DivisionFactor,
                                                    &dw_StopChannelMask,
                                                    &dw_StopCondition,
                                                    &dw_StopValue);
                    
                    /* Start the min/max acquisition */
                    
                    i_Error = soap_call_MX370x__TransducerInitAndStartMinMaxAcquisition(ps_SoapEnv,
                                                                                        pc_SOAPIPAddress,
                                                                                        SOAP_ACTION,
                                                                                        dw_TransducerTypeSelection,
                                                                                        dw_ChannelMask,
                                                                                        dw_DivisionFactor,
                                                                                        dw_StopChannelMask,
                                                                                        dw_StopCondition,
                                                                                        dw_StopValue,
                                                                                        0,
                                                                                        0,
                                                                                        0,
                                                                                        0,
                                                                                        &Response);
                    if (!i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, Response.syserrno, "soap_call_MX370x__TransducerInitAndStartMinMaxAcquisition", 0))
                       {
                       b_Exit = 0;
                       
                       do
                          {
                          i_Error = soap_call_MX370x__TransducerGetMinMaxStatus(ps_SoapEnv,
                                                                                pc_SOAPIPAddress,
                                                                                SOAP_ACTION,
                                                                                &dw_Dummy,
                                                                                &s_GetMinMaxStatusResponse);
						  if (!i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_GetMinMaxStatusResponse.iReturnValue, 0, "soap_call_MX370x__TransducerGetMinMaxStatus", 0))
                             {
                             printf("\n Min/Max Status : %lu Overflow:%lu",s_GetMinMaxStatusResponse.ulFlag,
                                                                           s_GetMinMaxStatusResponse.ulOverFlow);
                             
                             for (dw_Cpt = 0; dw_Cpt < dw_NbrOfChannelsMax;dw_Cpt ++)
                                {
                                if (dw_ChannelMask & (1<<dw_Cpt))
                                   {
                                   printf(" Channel %lu Min:%lX Max:%lX", dw_Cpt+1,
                                                                         s_GetMinMaxStatusResponse.pulMinValues[dw_Cpt],
                                                                         s_GetMinMaxStatusResponse.pulMaxValues[dw_Cpt]);
                                
                                   }
                                }

                             if (s_GetMinMaxStatusResponse.ulFlag == 2)
                                {
                                printf("\nCompare value reach !");
                                getch();
                                break;
                                }

                             if (s_GetMinMaxStatusResponse.ulOverFlow == 1)
                                {
                                printf("\nPLD Overflow !");
                                getch();
                                break;
                                }

                             }
                          else
                             {                                                         
                             b_Exit = 1;                          
                             }
                          if (kbhit ())
                             {
                             if (getch() == 27)
                                {
                                b_Exit = 1;
                             
                                }
                             }
							 
							/* 
							 * gSOAP doesn't release automatically the memory that 
							 * it allocates to deserialize data, that's why both 
							 * following functions have to be used to release the 
							 * allocated memory.
							 */
							soap_destroy(ps_SoapEnv);
							soap_end(ps_SoapEnv);
							
                          } while ( b_Exit == 0 );
                       i_Error = soap_call_MX370x__TransducerStopAndReleaseMinMaxAcquisition(ps_SoapEnv, 
                                                                                             pc_SOAPIPAddress, 
                                                                                             SOAP_ACTION, 
                                                                                             &dw_Dummy, 
                                                                                             &Response);
					   i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, Response.syserrno, "soap_call_MX370x__TransducerStopAndReleaseMinMaxAcquisition", 0);
                       }
                    }
                 }
              else
                 {
                 printf("Find the number of channel error\n");
              
                 }
              }

           /* Close the SOAP connection */
           i_SOAP_Close(ps_SoapEnv);
        
           }
        /**********************************/
        /*Print End programme informations*/
        /**********************************/
        
        printf("\n");
        printf("+---------------------------------------------------+\n");
        printf("| Program stopped                                   |\n");
        printf("+---------------------------------------------------+\n");
        printf("| Press any key to exit from sample                 |\n");
        printf("+---------------------------------------------------+\n");
        
        while (!(kbhit()))
           {
           }
	getch ();
        }
