/** @file Sequence.c
* Sequence : MX370x : Sequence acquisition
* 
*
* @author C. Guinot
* @date 27.03.2006
* 
* @version  $LastChangedRevision$
* @version $LastChangedDate$
* 
* 
* @par LICENCE
* @verbatim
*  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
*        
*        ADDI-DATA GmbH 
*        Airport Boulevard B210
*        D-77836 Rheinmnster Germany 
*        Tel: +49(0)7229/1847-0 
*        Fax: +49(0)7229/1847-200
*        http://www.addi-data-com 
*        info@addi-data.com 
*        
*
* @endverbatim   
*
* @par UPDATES                                                                                                          
* @verbatim                                                                                                             
*  Date      : 01.06.06                                                                                                 
*  Author        : C. Guinot                                                                                            
*  Description   : Remove the "_" character in the SOAP parameter name
*                                                                                                                       
*  Date      : 14.08.06                                                                                                 
*  Author        : C. Guinot                                                                                            
*  Description   : Add the data format management
*
* @endverbatim     
*
*
*/ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <io.h>
#include <math.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>


#include "..\..\..\..\Interface_Library\MX370x.nsmap"
#include "..\UTIL\utility.h"
#include "..\UTIL\socket.h"



/*******************************************************************************************************/
/*                                       UTILS Functions                                               */               
/*******************************************************************************************************/


INT i_UTILS_GetTheNumberOfChannels (PCHAR pc_ModuleType, DWORD * pdw_NbrOfChannels)
{
	DWORD dw_Cpt = 0;
	CHAR pc_NbrOfChannels [5];
	DWORD dw_NbrOfChannelIndex = 0;

	/* check presence of the substring "UNKNOWN" indicating a not yet configured module */
	if (strstr(pc_ModuleType,"UNKNOWN"))
	{
		fprintf(stderr,"! Remote module not configured\n");
		return -1;
	}

	/* If DIO system, the pattern is not the same */
	if (strstr(pc_ModuleType, "DIO"))
	{
		/* module is like: MSX-E3701-DIO-HB-XX or MSX-E3701-DIO-LVDT-XX*/	
		/* get a pointer to HB or LVDT */
		PCHAR beg = strstr(pc_ModuleType, "LVDT");
		if ( ! beg )
			beg = strstr(pc_ModuleType, "HB");
		if ( ! beg )
		{
			fprintf(stderr,"! Unknown system: %s \n", pc_ModuleType);
			return -1;
		}

		/* loop until we find the first "-" caracter */
		for (dw_Cpt = 0; dw_Cpt < strlen(beg);dw_Cpt++)
		{
			if (beg [dw_Cpt] == '-') 
			{
				dw_Cpt++;
				break;
			}
		}

		/* now we have the number of channels */
		for ( ; dw_Cpt < strlen(beg); dw_Cpt++)
		{
			if ( (beg [dw_Cpt] < '0') ||(beg [dw_Cpt] > '9') )
			{
				break;
			}
			pc_NbrOfChannels[dw_NbrOfChannelIndex] = beg[dw_Cpt];
			dw_NbrOfChannelIndex++;
		}

		/* check */
		if ( dw_NbrOfChannelIndex == 0 )
			return -1;

		/* convert and return */
		pc_NbrOfChannels[dw_NbrOfChannelIndex] = '\0';
		*pdw_NbrOfChannels = atol (pc_NbrOfChannels);
		return 0;
	}

	/* Normal systems */
	for (dw_Cpt = 0; dw_Cpt < strlen(pc_ModuleType);dw_Cpt++)
	{
		if (pc_ModuleType [dw_Cpt] == '-') 
		{
			dw_Cpt++;
			break;
		}
	}

	if (dw_Cpt < strlen(pc_ModuleType))
	{
		for (dw_NbrOfChannelIndex = 0; dw_NbrOfChannelIndex < (strlen(pc_ModuleType) - dw_Cpt ); dw_NbrOfChannelIndex ++)
		{
			if (pc_ModuleType [dw_Cpt + dw_NbrOfChannelIndex] == '-')
				break;
			else
				pc_NbrOfChannels [dw_NbrOfChannelIndex] = pc_ModuleType [dw_Cpt + dw_NbrOfChannelIndex];

		}
		pc_NbrOfChannels[dw_NbrOfChannelIndex] = '\0';

		if (dw_NbrOfChannelIndex < (strlen(pc_ModuleType) - dw_Cpt ))
			*pdw_NbrOfChannels = atol (pc_NbrOfChannels);
		else
			return -1;
	}
	else
		return -1;
	
	return 0;
}



int i_DisplayAndSelectTransducerType(struct soap *ps_SoapEnv,PCHAR pc_SOAPIPAddress,DWORD * pdw_TransducerTypeSelection)
{
	struct MX370x__unsignedlongResponse s_Response;
	struct MX370x__TransducerGetTypeInformationResponse TypeInfoResponse;

	INT i_Error = 0;
	DWORD dw_Cpt = 0;
	DWORD pdw_SelectionValueArray[100];
	DWORD dw_Choice = 0;
	DWORD dw_Dummy = 0;
  
	/* Get the number of transducer type */
	i_Error = soap_call_MX370x__TransducerGetNbrOfType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_Response);
	if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_Response.iReturnValue, 0, "soap_call_MX370x__TransducerGetNbrOfType", 0))
		return -1;
	else
	{
		printf("\nNumber of transducer types: %d\n", s_Response.ulValue);

		for (dw_Cpt = 0; dw_Cpt < s_Response.ulValue; dw_Cpt ++)
		{
			/* Get the transducer information */
			i_Error = soap_call_MX370x__TransducerGetTypeInformation (ps_SoapEnv,pc_SOAPIPAddress, SOAP_ACTION, dw_Cpt, &TypeInfoResponse);
			if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, TypeInfoResponse.iReturnValue, 0, "soap_call_MX370x__TransducerGetTypeInformation", 0))
				return -2;
			else
			{
				printf("\n");
				printf("Transducer Index: %d\n", dw_Cpt);
				printf("Selection Index: %d\n", TypeInfoResponse.ulTransducerSelectionIndex);

				pdw_SelectionValueArray[dw_Cpt] = TypeInfoResponse.ulTransducerSelectionIndex;
				printf("Name: %s\n", TypeInfoResponse.pcName);
				printf("Calibration status: %d\n", TypeInfoResponse.ulCalibrationStatus);
				printf("Type value: %d\n", TypeInfoResponse.ulType);
				printf("Frequency: %d Hz\n", TypeInfoResponse.ulFrequency);
				printf("Impedance: %d Ohm\n", TypeInfoResponse.ulImpedance);
				printf("Nominal Voltage: %lf V\n", TypeInfoResponse.dVeff);
				printf("Sensibility: %lf mV/V/mm\n", TypeInfoResponse.dSensibility);
				printf("Range: +/- %lf mm\n", TypeInfoResponse.dRange);
			}
		}

		do 
		{
			printf("\nEnter the index of the transducer type you want ot use (0 to %d): ", (s_Response.ulValue - 1));
			scanf("%d",&dw_Choice);
		}
		while (dw_Choice >= s_Response.ulValue);
		*pdw_TransducerTypeSelection = pdw_SelectionValueArray[dw_Choice];
	}
	return 0;
}

void GetSequenceParameters(DWORD dw_NbrOfChannelsMax, DWORD *pdw_NbrOfChannels, struct MX370x__unsignedLong16FixedArray *pdw_ChannelList, DWORD *pdw_DivisionFactor,
						   DWORD *pdw_NbrOfSequence, DWORD *pdw_NbrMaxSequenceToTransfer, DWORD *pdw_DelayMode, DWORD *pdw_DelayTimeUnit, DWORD *pdw_DelayTime,
						   DWORD *pdw_TriggerAction, DWORD *pdw_HardwareTriggerCount, DWORD *pdw_HardwareTriggerFilterTime, DWORD *pdw_ByTriggerNbrOfSeqToAcquire, DWORD *pdw_DataFormat)
{
	unsigned int i;
	DWORD dw_Choice = 0;

	*pdw_TriggerAction              = 0;
	*pdw_HardwareTriggerCount       = 0;
	*pdw_HardwareTriggerFilterTime  = 0;
	*pdw_ByTriggerNbrOfSeqToAcquire = 0;
	*pdw_DelayTimeUnit              = 0;
	*pdw_DelayTime                  = 0;
	*pdw_DataFormat                 = 0;


	do
	{
		printf("\nEnter the number of channels in sequence: ");
		scanf("%d", pdw_NbrOfChannels);
	}
	while ((*pdw_NbrOfChannels == 0) || (*pdw_NbrOfChannels > dw_NbrOfChannelsMax));

	for (i = 0; i < *pdw_NbrOfChannels; i++)
	{
		do
		{
			printf("\nEnter channel of sequence index %d: ", i+1);
			scanf("%d", &pdw_ChannelList->ulValue[i]);
		}
		while ((pdw_ChannelList->ulValue[i] > dw_NbrOfChannelsMax)||(pdw_ChannelList->ulValue[i] < 1));
		pdw_ChannelList->ulValue[i]--;
	}

	do
	{
		printf("\nEnter the division factor (5 to 255): ");
		scanf("%d", pdw_DivisionFactor);
	}
	while ((*pdw_DivisionFactor < 5) || (*pdw_DivisionFactor > 255));

	do
	{
		printf("\nEnter the number of sequences to acquire (0 = continuous): ");
		scanf("%d", pdw_NbrOfSequence);
	}
	while ((*pdw_NbrOfSequence < 0) || (*pdw_NbrOfSequence > 65535));

	do
	{
		printf("\nEnter the maximum number of sequence to acquire before a data transfer: ");
		scanf("%d", pdw_NbrMaxSequenceToTransfer);
	}
	while ((*pdw_NbrMaxSequenceToTransfer < 1) || (*pdw_NbrMaxSequenceToTransfer > 65535));

//	Delay
	do
	{
		printf("0. Delay not used\n");
		printf("1. Delay mode 1 used\n");
		printf("2. Delay mode 2 used\n");
		printf("Select the delay mode You want to use: ");
		scanf("%d", pdw_DelayMode);
	}
	while ((*pdw_DelayMode != 0) && (*pdw_DelayMode != 1) && (*pdw_DelayMode != 2));

	if (*pdw_DelayMode != 0)
	{
		do
		{
			printf("Select the delay timebase You want to use (0 = Miliseconds, 1 = Seconds): ");
			scanf("%d", pdw_DelayTimeUnit);
		}
		while ((*pdw_DelayTimeUnit != 0) && (*pdw_DelayTimeUnit != 1));

		do
		{
			printf("\nEnter the delay value: ");
			scanf("%d", pdw_DelayTime );
		}
		while ((*pdw_DelayTime  < 0) || (*pdw_DelayTime  > 65535));
	}

//	Trigger Start
	printf("\nDo You want to use the hardware trigger to start the acquisition (0 or 1)? ");
	scanf("%d", &dw_Choice);

	if (dw_Choice == 1)
	{
		do
		{
			printf("\n1. One shot trigger\n");
			printf("2. Sequence trigger\n");
			printf("Select the trigger mode do You want to use: ");
			scanf("%d", pdw_TriggerAction);
		}
		while ((*pdw_TriggerAction != 1) && (*pdw_TriggerAction != 2));

		do 
		{
			printf("\n1. rising edge\n");
			printf("2. falling edge\n");
			printf("3. both edges\n");
			printf("Select the active edge: ");
			scanf ("%d", &dw_Choice);
		}
		while ((dw_Choice < 1) || (dw_Choice > 3));

		*pdw_TriggerAction = *pdw_TriggerAction | (dw_Choice << 6);

		do
		{
			printf("\nEnter the number of edges before the trigger action occur (1 to 65535): ");
			scanf("%d",pdw_HardwareTriggerCount);
		}
		while ((*pdw_HardwareTriggerCount == 0) || (*pdw_HardwareTriggerCount > 65535));

		do
		{
			printf("\nEnter the filter time (multiplier from 250 ns step) for the input trigger (0 = disable, max : 65535 -> 16,383750 ms): ");
			scanf("%d",pdw_HardwareTriggerFilterTime);
		}
		while (*pdw_HardwareTriggerFilterTime > 65535);
	}
	else
	{
		printf("\nDo You want to use the synchronisation trigger to start the acquisition (0 or 1)? ");
		scanf("%d", &dw_Choice);

		if (dw_Choice == 1)
		{
			do
			{
				printf("\n1. One shot trigger\n");
				printf("2. Sequence trigger\n");
				printf("Select the trigger mode do You want to use: ");
				scanf("%d", &dw_Choice);
			}
			while ((dw_Choice != 1) && (dw_Choice != 2));

		    *pdw_TriggerAction = dw_Choice << 8;
		}
	}


	if (((*pdw_TriggerAction & 2) == 0x2) || ((*pdw_TriggerAction & 0x200) == 0x200))
	{
		do
		{
			printf("\nEnter the number of sequence to acquire by each trigger: ");
			scanf("%d", pdw_ByTriggerNbrOfSeqToAcquire);
		}
		while (*pdw_ByTriggerNbrOfSeqToAcquire == 0);
	}

//	Trigger Stop
	if ((*pdw_TriggerAction == 0) || ((*pdw_TriggerAction & 0x101) != 0))
	{
		printf("\nDo You want to use the hardware trigger to stop the acquisition (0 or 1)? ");
		scanf("%d", &dw_Choice);

		if (dw_Choice == 1)
		{
			*pdw_TriggerAction = *pdw_TriggerAction | (1 << 16); 

			do 
			{
				printf("\n1. rising edge\n");
				printf("2. falling edge\n");
				printf("3. both edges\n");
				printf("Select the active edge: ");
				scanf ("%d", &dw_Choice);
			}
			while ((dw_Choice < 1) || (dw_Choice > 3));

			*pdw_TriggerAction = *pdw_TriggerAction | (dw_Choice << 17);

			do
			{
				printf("\nDo you want to use the number of edge before the trigger occur (0 or 1): ");
				scanf("%d",&dw_Choice);
			}
			while (dw_Choice > 1);

			if (dw_Choice == 1)
			{
				*pdw_TriggerAction = *pdw_TriggerAction | (1 << 19);

				if (*pdw_HardwareTriggerCount == 0)
				{
					do
					{
						printf("\nEnter the number of edges before the trigger action occur (1 to 65535): ");
						scanf("%d",pdw_HardwareTriggerCount);
					}
					while ((*pdw_HardwareTriggerCount == 0) || (*pdw_HardwareTriggerCount > 65535));
				}
			}

		}
	}



// Data format
	*pdw_DataFormat = 0;

	do 
	{
	    printf("\nDo You want to receive an absolut time stamp (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = dw_Choice;

	if (*pdw_DataFormat == 0)
	{

		do 
		{
			printf("\nDo You want to receive a relativ time stamp (0: No, 1: Yes) ?");
			scanf ("%d",&dw_Choice);
		}
		while (dw_Choice > 1);

		*pdw_DataFormat = dw_Choice << 4;
	}


	do 
	{
	    printf("\nDo You want to receive a Sequence counter (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 1);

	do 
	{
	    printf("\nDo You want to receive the value in digital form (tape 0) or in analog form mm (type 1) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 2);

	do 
	{
	    printf("\nDo You want to invert the value sign (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 3);


	do 
	{
	    printf("\nDo You want to receive the hardware trigger input information ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 5);
}


#define MAX_SOCKET_DATA_FIFO 100

typedef struct
      {
      BYTE      b_Exit;
      INT       i_Handle;
      DWORD    dw_DataFormat;
      DWORD    dw_NumberOfChannelsToAcquire;
	  DWORD    dw_NbrMaxSequenceToTransfer;
	  DWORD    dw_SingelSequenceSize;
	  DWORD    dw_SocketDataSize;
      SOCKET    s_ClientSocket;
	  HANDLE    h_EventHandle;
	  struct
	     {
	     BYTE  *  pb_SocketData [MAX_SOCKET_DATA_FIFO];
		 DWORD    dw_ReadPointer;
		 DWORD    dw_WritePointer;
		 BYTE      b_Overflow;
		 }s_SocketDataFIFO;
      }str_ThreadInfo;


DWORD WINAPI dw_ReadSocketThreadProc (LPVOID lpParameter)
        {
        str_ThreadInfo *  ps_ThreadInfo;
		INT                i_ReturnValue        = 0;
    	DWORD             dw_RequestValue       = 0;
    	DWORD             dw_NbrOfReceiveValues = 0;
        
        /**************************
        * Variable initialisation 
        ***************************/
        ps_ThreadInfo = (str_ThreadInfo *) lpParameter;
        
        do
        {
	        /************************************
		     * Test if no internal FIFO overflow 
			 ************************************/

		    if (((ps_ThreadInfo->s_SocketDataFIFO.dw_WritePointer + 1) % MAX_SOCKET_DATA_FIFO) != ps_ThreadInfo->s_SocketDataFIFO.dw_ReadPointer)
			{
				/* Calculate the read data size */
				dw_RequestValue         = ps_ThreadInfo->dw_SocketDataSize;
				dw_NbrOfReceiveValues   = 0;

				do
				{
					i_ReturnValue = recv (ps_ThreadInfo->s_ClientSocket, &ps_ThreadInfo->s_SocketDataFIFO.pb_SocketData[ps_ThreadInfo->s_SocketDataFIFO.dw_WritePointer][dw_NbrOfReceiveValues], dw_RequestValue, 0); 

					if (i_ReturnValue != SOCKET_ERROR)
					{
						dw_NbrOfReceiveValues = dw_NbrOfReceiveValues + i_ReturnValue;
						dw_RequestValue = dw_RequestValue - i_ReturnValue;
					}
				}
				while ((dw_NbrOfReceiveValues < ps_ThreadInfo->dw_SocketDataSize) && (i_ReturnValue != SOCKET_ERROR));


				if (i_ReturnValue != SOCKET_ERROR)
				{  
					/************************************
					 * Increment the write FIFO pointer *
					 ************************************/

					ps_ThreadInfo->s_SocketDataFIFO.dw_WritePointer = (ps_ThreadInfo->s_SocketDataFIFO.dw_WritePointer + 1) % MAX_SOCKET_DATA_FIFO;

					/**************************
					 * Set the reeceive event *
					 **************************/
					SetEvent (ps_ThreadInfo->h_EventHandle);
				}
				else
				{	
				    /*****************************/
					/* Abnormal error, print it. */
					/*****************************/		
					if (ps_ThreadInfo->b_Exit != 2)
						printf("Socket Error %d\n", WSAGetLastError());

        			ps_ThreadInfo->b_Exit = 1;
					SetEvent (ps_ThreadInfo->h_EventHandle);

				}
			}
			else
			{
				printf ("\nInternal sample FIFO overflow. This stop the sample\n");
				ps_ThreadInfo->b_Exit = 1;
				ps_ThreadInfo->s_SocketDataFIFO.b_Overflow = 1;
				SetEvent (ps_ThreadInfo->h_EventHandle);
			}

        } 
        while (ps_ThreadInfo->b_Exit == 0);


        /*********************
        * Return from thread 
        **********************/
        return (0);

        }


DWORD WINAPI dw_SocketDataManagementThreadProc (LPVOID lpParameter)
        {
        str_ThreadInfo *  ps_ThreadInfo;
    	BYTE               b_Cpt                = 0;
		BYTE               b_ReadDataIndex      = 0;
		INT                i_ReturnValue        = 0;
        DWORD             dw_SeqCounter         = 0;
		DWORD             dw_SequenceCounter    = 0;
        CHAR               c_Write   [300];
		DWORD           *pdw_SequenceData;
		float            *pf_RealValue;
        
        /**************************
        * Variable initialisation 
        ***************************/
        ps_ThreadInfo = (str_ThreadInfo *) lpParameter;
        
        do
        {

			WaitForSingleObject (ps_ThreadInfo->h_EventHandle, INFINITE);

			while ((ps_ThreadInfo->s_SocketDataFIFO.dw_ReadPointer != ps_ThreadInfo->s_SocketDataFIFO.dw_WritePointer) && (ps_ThreadInfo->b_Exit != 2))
			{
				for (dw_SequenceCounter = 0, (BYTE *) pdw_SequenceData = ps_ThreadInfo->s_SocketDataFIFO.pb_SocketData[ps_ThreadInfo->s_SocketDataFIFO.dw_ReadPointer];
				     (dw_SequenceCounter < ps_ThreadInfo->dw_NbrMaxSequenceToTransfer) && (ps_ThreadInfo->b_Exit != 2);
					 dw_SequenceCounter ++, pdw_SequenceData = pdw_SequenceData + (ps_ThreadInfo->dw_SingelSequenceSize) / sizeof (DWORD))
				{ 
					c_Write [0]     = 0;
					b_ReadDataIndex = 0;

					/* Test if the data format absolut time stamp is activated */
					if (ps_ThreadInfo->dw_DataFormat & 1)
					{
				        sprintf(c_Write,"%s %s",c_Write, ConvertTimeStamp (pdw_SequenceData [b_ReadDataIndex + 0],pdw_SequenceData [b_ReadDataIndex + 1]));
						b_ReadDataIndex = b_ReadDataIndex + 2;
					}

					/* Test if the data format relativ time stamp is activated */
					if (ps_ThreadInfo->dw_DataFormat & 0x10)
					{
				        sprintf(c_Write,"%s %03lu s %03lu ms %03lu us ",c_Write, pdw_SequenceData [b_ReadDataIndex + 0],(DWORD)(pdw_SequenceData [b_ReadDataIndex + 1] / 1000), (DWORD)(pdw_SequenceData [b_ReadDataIndex + 1] % 1000));
						b_ReadDataIndex = b_ReadDataIndex + 2;
					}

					/* Test if sequence counter data is activated */
					if (ps_ThreadInfo->dw_DataFormat & 2)
					{
				        sprintf(c_Write,"%s Sequence counter = %lu", c_Write, pdw_SequenceData [b_ReadDataIndex + 0]);
						b_ReadDataIndex = b_ReadDataIndex + 1;
					}

					/* Test if hardware trigger  information is activated */
					if (ps_ThreadInfo->dw_DataFormat & 0x20)
					{
				        sprintf(c_Write,"%s Hardware trigger info = 0x%lX", c_Write, pdw_SequenceData [b_ReadDataIndex + 0]);
						b_ReadDataIndex = b_ReadDataIndex + 1;
					}

					/* Test if receive digital value */
					if ((ps_ThreadInfo->dw_DataFormat & 4) == 0x0) 
					{
					    for (b_Cpt = 0; b_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; b_Cpt ++)
					    { 
						    sprintf(c_Write,"%s %d:0x%lX",c_Write,b_Cpt+1, pdw_SequenceData [b_ReadDataIndex + b_Cpt]);
					    }
				    }
					else
					{
					    pf_RealValue = (float *) &pdw_SequenceData [b_ReadDataIndex];

					    for (b_Cpt = 0; b_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; b_Cpt ++)
					    { 
                        sprintf(c_Write,"%s %d:%fmm",c_Write,b_Cpt,pf_RealValue[b_Cpt]);
					    }
					}

					sprintf(c_Write,"%s\n", c_Write);
					printf (c_Write);

					/*****************
					 * Test if store *
					 *****************/

					if (ps_ThreadInfo->i_Handle != -1)
					{
						write (ps_ThreadInfo->i_Handle, c_Write, strlen (c_Write));
					}
				}
			
				/***********************************
				 * Increment the read FIFO pointer *
				 ***********************************/

				ps_ThreadInfo->s_SocketDataFIFO.dw_ReadPointer = (ps_ThreadInfo->s_SocketDataFIFO.dw_ReadPointer + 1) % MAX_SOCKET_DATA_FIFO;
			}

			if (ps_ThreadInfo->b_Exit)
			{
				ps_ThreadInfo->b_Exit = 2;
			}
        } 
        while (ps_ThreadInfo->b_Exit == 0);

        /*********************
        * Return from thread 
        **********************/
        return (0);

        }



void main(void)
{
	CHAR								     c_IPAddress    [50];
	CHAR								     c_IPAddressTemp[50];
	CHAR								     c_SOAPIPAddress[50];
    INT		                                 i_key		  			      = 0;
	INT										 i_Error                      = 0;
    INT										 i_Handle                     = -1;
	INT										 i_CurrentPriority            = 0;
	INT                                      i_ReturnValue                = 0;
	DWORD                                   dw_Dummy                      = 0;
	DWORD                                   dw_Choice                     = 0;
    DWORD                                   dw_ThreadId                   = 0;
	DWORD                                   dw_NbrOfChannelsMax           = 16;
	DWORD                                   dw_SockedBufferCounter        = 0;
	xsd__unsignedLong                       dw_TransducerTypeSelection    = 0;
	xsd__unsignedLong                       dw_NbrOfChannels              = 0;
	xsd__unsignedLong                       dw_DivisionFactor             = 0;
	xsd__unsignedLong                       dw_NbrOfSequence              = 0;
	xsd__unsignedLong                       dw_NbrMaxSequenceToTransfer   = 0;
	xsd__unsignedLong                       dw_DelayMode                  = 0;
	xsd__unsignedLong                       dw_DelayTimeUnit              = 0;
	xsd__unsignedLong                       dw_DelayValue                 = 0;
	xsd__unsignedLong                       dw_TriggerAction              = 0;
	xsd__unsignedLong                       dw_HardwareTriggerCount       = 0;
	xsd__unsignedLong                       dw_HardwareTriggerFilterTime  = 0;
	xsd__unsignedLong                       dw_ByTriggerNbrOfSeqToAcquire = 0;
	xsd__unsignedLong                       dw_DataFormat                 = 0;
	xsd__unsignedLong                       dw_ReservedParameter2         = 0;
	xsd__unsignedLong                       dw_ReservedParameter3         = 0;
	xsd__unsignedLong                       dw_ReservedParameter4         = 0;
	struct MXCommon__ByteArrayResponse       s_ByteArrayResponse;
	struct MX370x__unsignedLong16FixedArray dw_ChannelList;
	struct MX370x__Response                 Response;
	struct soap                            *ps_SoapEnv = NULL;
	SOCKET                                   s_ClientSocket;
    str_ThreadInfo                           s_ThreadInfo;
    HANDLE                                   h_ReadSocketThreadHandle     = 0;
    HANDLE                                   h_DataManagementThreadHandle = 0;


	/***************************/
    /* Variable initialisation */
	/***************************/

	memset (&s_ThreadInfo.s_SocketDataFIFO, 0, sizeof (s_ThreadInfo.s_SocketDataFIFO));

	/**************************/
	/* Get the console handle */
	/**************************/
	ShowWindow(GetForegroundWindow (), SW_SHOWMAXIMIZED);

	/****************************/
	/* Print sample information */
	/****************************/
	printf("+---------------------------------------------------+\n");
	printf("| MX370x Sequence                                   |\n");
	printf("| MX370x Sequence Test Program                      |\n");
	printf("+---------------------------------------------------+\n");
	printf("| <ESC> exit sample                                 |\n");
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(c_IPAddressTemp);

		v_UTILS_ClearNullInIPAddress(c_IPAddressTemp,c_IPAddress);
		sprintf(c_SOAPIPAddress,"%s:5555", c_IPAddress);
		printf("SOAP IP Address: %s\n", c_SOAPIPAddress);

		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv,c_SOAPIPAddress , SOAP_ACTION, &dw_Dummy, &s_ByteArrayResponse);
		if (!i_CheckAndPrintError (ps_SoapEnv, c_SOAPIPAddress, SOAP_ACTION, i_Error, s_ByteArrayResponse.sResponse.iReturnValue, 0, "soap_call_MXCommon__GetModuleType", 0))
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);
			if (i_UTILS_GetTheNumberOfChannels(s_ByteArrayResponse.sArray.__ptr, &dw_NbrOfChannelsMax) == 0)
			{
				printf("Number of transducer channels: %d \n", dw_NbrOfChannelsMax);
				/* Get Transducer type selection */
				if (i_DisplayAndSelectTransducerType(ps_SoapEnv, c_SOAPIPAddress, (DWORD *)&dw_TransducerTypeSelection) == 0)
				{
					/* Get the auto refresh initialisation parameter */
					GetSequenceParameters(	dw_NbrOfChannelsMax,
											&dw_NbrOfChannels,
											&dw_ChannelList,
											&dw_DivisionFactor,
											&dw_NbrOfSequence,
											&dw_NbrMaxSequenceToTransfer,
											&dw_DelayMode,
											&dw_DelayTimeUnit,
											&dw_DelayValue,
											&dw_TriggerAction,
											&dw_HardwareTriggerCount,
											&dw_HardwareTriggerFilterTime,
											&dw_ByTriggerNbrOfSeqToAcquire,
                                            &dw_DataFormat);

					s_ThreadInfo.b_Exit                       = 0;
		            s_ThreadInfo.dw_DataFormat                = dw_DataFormat;
		            s_ThreadInfo.dw_NumberOfChannelsToAcquire = dw_NbrOfChannels;
					s_ThreadInfo.dw_NbrMaxSequenceToTransfer  = dw_NbrMaxSequenceToTransfer;
				
					/**********************************************/
					/* Calculate the reading the socket data size */
					/**********************************************/

					s_ThreadInfo.dw_SingelSequenceSize = (dw_NbrOfChannels + ((dw_DataFormat & 1) * 2)+ (((dw_DataFormat>>4) & 1) * 2) + ((dw_DataFormat & 2) >> 1) + ((dw_DataFormat & 0x20) >> 5)) * sizeof (DWORD);
					s_ThreadInfo.dw_SocketDataSize     = s_ThreadInfo.dw_SingelSequenceSize * dw_NbrMaxSequenceToTransfer;
											
					/* Initialise the socket to receive the data */
					if (i_Socket_ClientOpen(c_IPAddress, 8989, &s_ClientSocket) == 0)
					{

						do 
						{
							printf("\nDo You want to store the sequence data (0:No, 1:Yes) ?");
							scanf ("%d",&dw_Choice);
						}
						while (dw_Choice > 1);

						/* Test if store */

						if (dw_Choice)
						{
							i_Handle = open ("Sequence.txt", O_CREAT | O_TRUNC | O_RDWR,S_IREAD | S_IWRITE);
						}

		                if (((i_Handle != -1) && (dw_Choice == 1)) || (dw_Choice == 0))
				        {
							/*************************************************/
							/* Memory allocation for reading the socket data */
							/*************************************************/

							for (dw_SockedBufferCounter = 0; dw_SockedBufferCounter < MAX_SOCKET_DATA_FIFO; dw_SockedBufferCounter ++)
							{
								if ((s_ThreadInfo.s_SocketDataFIFO.pb_SocketData[dw_SockedBufferCounter] = malloc (s_ThreadInfo.dw_SocketDataSize)) == NULL)
								{
									break;
								}
							}


							if (dw_SockedBufferCounter == MAX_SOCKET_DATA_FIFO)
							{
								/**************************/
								/* Save the client socket */
								/**************************/

					            s_ThreadInfo.s_ClientSocket = s_ClientSocket;
				                s_ThreadInfo.i_Handle       = i_Handle;

								/********************************
								 * Thread synchronisation event *
								 ********************************/

								s_ThreadInfo.h_EventHandle = CreateEvent (NULL, FALSE, FALSE, "MSX-E370x sequence");

								/**************************************
								 * Create the read socked data thread *
								 **************************************/

								h_ReadSocketThreadHandle = CreateThread (NULL,
								 	        		                     0,
											   	                         dw_ReadSocketThreadProc,
												                         &s_ThreadInfo,
														                 0,
																         &dw_ThreadId);

								/*************************************************************************************************/
								/* Change the process priority to the maximum for the childs to have maximum OS allowed priority */
								/*************************************************************************************************/

								i_CurrentPriority = GetThreadPriority (GetCurrentThread());
								SetThreadPriority (GetCurrentThread(), THREAD_PRIORITY_TIME_CRITICAL);

								SetThreadPriority (h_ReadSocketThreadHandle, THREAD_PRIORITY_TIME_CRITICAL);

								/********************************/
								/* Restore the process priority */
								/********************************/
						
								SetThreadPriority (GetCurrentThread(), i_CurrentPriority);

								Sleep (100);


								/********************************************
								 * Create the socked data management thread *
								 ********************************************/

								h_DataManagementThreadHandle = CreateThread (NULL,
								 	        		                         0,
											   	                             dw_SocketDataManagementThreadProc,
												                             &s_ThreadInfo,
												                             0,
												                             &dw_ThreadId);

								SetThreadPriority (h_DataManagementThreadHandle, THREAD_PRIORITY_TIME_CRITICAL);

						        ps_SoapEnv->send_timeout    = 5;
								ps_SoapEnv->recv_timeout    = 5;
								ps_SoapEnv->accept_timeout  = 5;

								/*************************************/
								/* Initialise and start the sequence */
								/*************************************/

								i_Error = soap_call_MX370x__TransducerInitAndStartSequence(ps_SoapEnv,
																						   c_SOAPIPAddress,
																						   SOAP_ACTION, 
																						   dw_TransducerTypeSelection,
																						   dw_NbrOfChannels,
																						   &dw_ChannelList,
																						   dw_DivisionFactor,
																						   dw_NbrOfSequence,
																						   dw_NbrMaxSequenceToTransfer,
																						   dw_DelayMode,
																						   dw_DelayTimeUnit,
																						   dw_DelayValue,
																						   dw_TriggerAction,
																						   dw_HardwareTriggerCount,
																						   dw_HardwareTriggerFilterTime,
																						   dw_ByTriggerNbrOfSeqToAcquire,
																						   dw_DataFormat,
																						   dw_ReservedParameter2,
																						   dw_ReservedParameter3,
																						   dw_ReservedParameter4,
																						   &Response);

								if (i_CheckAndPrintError (ps_SoapEnv, c_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, Response.syserrno, "soap_call_MX370x__TransducerInitAndStartSequence", 0))
								{
									s_ThreadInfo.b_Exit = 1;
								    Sleep (100);
								}
								else
								{
									/* 
									 * gSOAP doesn't release automatically the memory that 
									 * it allocates to deserialize data, that's why both 
									 * following functions have to be used to release the 
									 * allocated memory.
									 */
									soap_destroy(ps_SoapEnv);
									soap_end(ps_SoapEnv);
							
				                    printf("\n Press ESC to exit the sample...\n");

			                        do
				                    {
								        if (kbhit()) 
								        {
										    if (getch() == 27)
											{
												s_ThreadInfo.b_Exit = 2;
											}
								        }
										else
										{
											Sleep (100);
										}
								    }
				                    while (s_ThreadInfo.b_Exit != 2);

									Sleep (100);

									/*********************************
									 * Test if internal FIFO overflow
									 *********************************/

									if (s_ThreadInfo.s_SocketDataFIFO.b_Overflow)
									{
										printf ("\nInternal sample FIFO overflow. This have stop the sample\n");
									}

		
					                /**********************
					                 * Stop the acquisition 
					                 **********************/

									i_Error = soap_call_MX370x__TransducerStopAndReleaseSequence(ps_SoapEnv, c_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &Response);
									i_CheckAndPrintError (ps_SoapEnv, c_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, Response.syserrno, "soap_call_MX370x__TransducerStopAndReleaseSequence", 0);
							    }                    
							}
							else
							{
								printf ("Socket memory allocation error\n");
							}

				        }
						else
		                {
				            printf ("Sequence.txt file creation error \n");
		                }

						/* Close the socket connection */
						i_Socket_ClientClose(s_ClientSocket);
					}
					else
					{
						printf("i_Socket_ClientOpen Error\n");
					}

				}
			}
			else
			{
				printf("find the number of channel error\n");
			}
		}
	}

	/******************************
	 * Test if SOAP connection open
	 ******************************/

	if (ps_SoapEnv != NULL)
	{
	    /****************************
		 * Close the SOAP connection 
		 ****************************/
		i_SOAP_Close(ps_SoapEnv);
	}

    /********************
     * Test if file open 
     ********************/
	if (i_Handle != -1)
	{
		/*****************
		 * Close the file 
	     *****************/
		close (i_Handle);
	}


    /***************************
     * Test if thread installed 
     ****************************/
    if (h_ReadSocketThreadHandle != 0)
	{
		/*************************
		 * Close the thread handle 
	     **************************/
		CloseHandle (h_ReadSocketThreadHandle);
	}

    /***************************
     * Test if thread installed 
     ****************************/
    if (h_DataManagementThreadHandle != 0)
	{
		/*************************
		 * Close the thread handle 
	     **************************/
		CloseHandle (h_DataManagementThreadHandle);
	}

	/****************************
	 * Free the allocated memory
	 ****************************/

	for (dw_SockedBufferCounter = 0; dw_SockedBufferCounter < MAX_SOCKET_DATA_FIFO; dw_SockedBufferCounter ++)
	{
		/**********************************
		 * Test if socket memory allocated 
		 **********************************/
		
		if (s_ThreadInfo.s_SocketDataFIFO.pb_SocketData[dw_SockedBufferCounter] != NULL)
		{
			free (s_ThreadInfo.s_SocketDataFIFO.pb_SocketData[dw_SockedBufferCounter]);
		}
	}
   
	/**********************************/
	/*Print End programme informations*/
	/**********************************/
	printf("\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));

}
