/** @file Socket.c
* Socket Functions 
* 
*
* @author C. Guinot
* @date 30.01.2006
* 
* @version  $LastChangedRevision$
* @version $LastChangedDate$
* 
* 
* @par LICENCE
* @verbatim
*  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
*        
*        ADDI-DATA GmbH 
*        Airport Boulevard B210
*        D-77836 Rheinmnster Germany 
*        Tel: +49(0)7229/1847-0 
*        Fax: +49(0)7229/1847-200
*        http://www.addi-data-com 
*        info@addi-data.com 
*        
*
* @endverbatim   
*
*
*/ 
#ifndef _WIN32
   #pragma warn -par
#endif

#include <stdio.h>
#include <math.h>
#include <conio.h>
#include <STDARG.H>

#include "define.h"

/** \internal 
* Open a UDP client socket
* @param[in] w_LocalPort: Local port number to use
* @param[out] ps_ClientSocket : socket descriptor
*
* @return ADDIDATA_SOCKET_WSASTARTUP_ERROR <br>
*         ADDIDATA_SOCKET_GETHOSTBYNAME_ERROR <br>
*         ADDIDATA_SOCKET_CONNECT_ERROR <br>
*/
INT i_Socket_ClientOpenUDP (WORD w_LocalPort, SOCKET  *ps_ClientSocket)
{
	INT result = 0;
	WSADATA wsaData;
	struct sockaddr_in rAddr;
	
	/* initialise Winsock */
	result = WSAStartup(MAKEWORD(2, 2), &wsaData);
	if ( result != NO_ERROR)
	{
		printf("WSAStartup error: %d\n", result);
		return(ADDIDATA_SOCKET_WSASTARTUP_ERROR);
	}

	/* create the socket receiver */
	*ps_ClientSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if ( (*ps_ClientSocket) == INVALID_SOCKET )
	{
		printf("socket failed with error: %d\n", WSAGetLastError());
		WSACleanup();
		return(ADDIDATA_CREATE_SOCKET_ERROR);
	}

	/* bind the socket */
	rAddr.sin_family = AF_INET;
	rAddr.sin_port = htons(w_LocalPort);
	rAddr.sin_addr.S_un.S_addr = htonl(INADDR_ANY);

	result = bind((*ps_ClientSocket), (SOCKADDR*)&rAddr, sizeof(rAddr));
	if ( result != 0 )
	{
		printf("bind failed with error: %d\n", WSAGetLastError());
		WSACleanup();
		return(ADDIDATA_SOCKET_CONNECT_ERROR);   
	}
	
	return 0;
}



/** \internal 
* Open a client socket
*
* @param[in] pc_ServerIPAddress : IP Address of the server
* @param[in] w_ServerPort : Port number of the server
* @param[out] ps_ClientSocket : socket descriptor
*
* @return ADDIDATA_SOCKET_WSASTARTUP_ERROR <br>
*         ADDIDATA_SOCKET_GETHOSTBYNAME_ERROR <br>
*         ADDIDATA_CREATE_SOCKET_ERROR <br>
*         ADDIDATA_SOCKET_CONNECT_ERROR <br>
*/
INT i_Socket_ClientOpen (PCHAR pc_ServerIPAddress,WORD w_ServerPort,SOCKET  *ps_ClientSocket)
   {

   struct hostent *hp;
   BYTE b_Error = 0;   
   struct sockaddr_in sai_Server;
   WSADATA wsaData;
   INT i_ReturnValue = 0;

   /*********************************************/
   /* Initiates use of Ws2_32.dll by a process. */
   /*********************************************/
   if (WSAStartup (0x202,&wsaData) == SOCKET_ERROR)
      {
      //sprintf(outtext, "WSAStartup failed with error %d", WSAGetLastError());                                         
      //MessageBox(outtext, "WINSOCK CLIENT ERROR", MB_OK);  
      WSACleanup();
      return(ADDIDATA_SOCKET_WSASTARTUP_ERROR);
      }
   
   /*********************************/
   /* Open a file to save the value */
   /*********************************/
   hp = gethostbyname(pc_ServerIPAddress);
   
   if (hp == NULL )
      {
      //sprintf(outtext,"Client: Cannot resolve address [%s]: Error %d", pc_ServerAddress, WSAGetLastError());                    
      //MessageBox(outtext, "WINSOCK CLIENT ERROR", MB_OK);          
      WSACleanup();
      return(ADDIDATA_SOCKET_GETHOSTBYNAME_ERROR);
      }
   
   /****************************************************************/
   /* Copy the resolved information into the sockaddr_in structure */
   /****************************************************************/
   memset(&sai_Server,0,sizeof(sai_Server));
   memcpy(&(sai_Server.sin_addr),hp->h_addr,hp->h_length);
   sai_Server.sin_family = AF_INET;
   sai_Server.sin_port = htons(w_ServerPort);
   
   /*********************/
   /* Create the socket */
   /*********************/
   /* Open a socket       */
   *ps_ClientSocket = socket(AF_INET,SOCK_STREAM,0);
   
   if (*ps_ClientSocket <0)
      {
      
      //sprintf(outtext,"Client: Error Opening socket: Error %d",WSAGetLastError());
      //MessageBox(outtext, "WINSOCK CLIENT ERROR", MB_OK);  
      WSACleanup();
      return(ADDIDATA_CREATE_SOCKET_ERROR);
   
      }

   /************************************/
   /* Connect the client on the server */
   /************************************/
   if (connect(*ps_ClientSocket,(struct sockaddr*)&sai_Server,sizeof(struct sockaddr_in))== SOCKET_ERROR)
      {
      //sprintf(outtext,"connect() failed: %d",WSAGetLastError());
      //MessageBox(outtext, "WINSOCK CLIENT ERROR", MB_OK);  
      
      WSACleanup();
      return(ADDIDATA_SOCKET_CONNECT_ERROR);
   
      }

   return(0);
   }

/** \internal 
* Open a client socket
*
* @param[in] pc_ServerIPAddress : IP Address of the server
* @param[in] w_ServerPort : Port number of the server
* @param[in] i_ReceiveBufferSize : Receive buffer size (byte)
* @param[out] ps_ClientSocket : socket descriptor
*
* @return ADDIDATA_SOCKET_WSASTARTUP_ERROR <br>
*         ADDIDATA_SOCKET_GETHOSTBYNAME_ERROR <br>
*         ADDIDATA_CREATE_SOCKET_ERROR <br>
*         ADDIDATA_SOCKET_RCVBUF_ERROR <br>
*         ADDIDATA_SOCKET_CONNECT_ERROR <br>
*/
INT i_Socket_ClientOpenEx (PCHAR pc_ServerIPAddress,WORD w_ServerPort,INT i_ReceiveBufferSize, SOCKET  *ps_ClientSocket)
   {

   struct hostent *hp;
   BYTE b_Error = 0;   
   struct sockaddr_in sai_Server;
   WSADATA wsaData;
   INT i_ReturnValue = 0;
   BOOL bo_Test = TRUE;

   /*********************************************/
   /* Initiates use of Ws2_32.dll by a process. */
   /*********************************************/
   if (WSAStartup (0x202,&wsaData) == SOCKET_ERROR)
      {
      //sprintf(outtext, "WSAStartup failed with error %d", WSAGetLastError());                                         
      //MessageBox(outtext, "WINSOCK CLIENT ERROR", MB_OK);  
      WSACleanup();
      return(ADDIDATA_SOCKET_WSASTARTUP_ERROR);
      }
   
   /*********************************/
   /* Open a file to save the value */
   /*********************************/
   hp = gethostbyname(pc_ServerIPAddress);
   
   if (hp == NULL )
      {
      //sprintf(outtext,"Client: Cannot resolve address [%s]: Error %d", pc_ServerAddress, WSAGetLastError());                    
      //MessageBox(outtext, "WINSOCK CLIENT ERROR", MB_OK);          
      WSACleanup();
      return(ADDIDATA_SOCKET_GETHOSTBYNAME_ERROR);
      }
   
   /****************************************************************/
   /* Copy the resolved information into the sockaddr_in structure */
   /****************************************************************/
   memset(&sai_Server,0,sizeof(sai_Server));
   memcpy(&(sai_Server.sin_addr),hp->h_addr,hp->h_length);
   sai_Server.sin_family = AF_INET;
   sai_Server.sin_port = htons(w_ServerPort);
   
   /*********************/
   /* Create the socket */
   /*********************/
   /* Open a socket       */
   *ps_ClientSocket = socket(AF_INET,SOCK_STREAM,0);
   
   if (*ps_ClientSocket <0)
      {
      
      //sprintf(outtext,"Client: Error Opening socket: Error %d",WSAGetLastError());
      //MessageBox(outtext, "WINSOCK CLIENT ERROR", MB_OK);  
      WSACleanup();
      return(ADDIDATA_CREATE_SOCKET_ERROR);
   
      }

   /****************************************/
   /* Set the socket reception buffer size */
   /****************************************/


   if (setsockopt (*ps_ClientSocket, SOL_SOCKET, SO_RCVBUF, (char *) &i_ReceiveBufferSize, sizeof (i_ReceiveBufferSize)) == SOCKET_ERROR)
      {
      WSACleanup();
      return(ADDIDATA_SOCKET_RCVBUF_ERROR);
	  }

   /************************************/
   /* Connect the client on the server */
   /************************************/
   if (connect(*ps_ClientSocket,(struct sockaddr*)&sai_Server,sizeof(struct sockaddr_in))== SOCKET_ERROR)
      {
      //sprintf(outtext,"connect() failed: %d",WSAGetLastError());
      //MessageBox(outtext, "WINSOCK CLIENT ERROR", MB_OK);  
      
      WSACleanup();
      return(ADDIDATA_SOCKET_CONNECT_ERROR);
   
      }

   return(0);
   }


/** \internal 
* Close a client socket
*
* @param[in] s_ClientSocket : socket descriptor
*
* @return -
*/
INT i_Socket_ClientClose (SOCKET  s_ClientSocket)
   {
   INT i_ReturnValue = 0;
   closesocket (s_ClientSocket);

   WSACleanup();
   return(i_ReturnValue);
   }

